@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -89,6 +89,22 @@ static void path_helper(Symbol *symbol, QList<const Name *> *names)
     }
 }
 
+static bool isNestedInstantiationEnclosingTemplate(
+        ClassOrNamespace *nestedClassOrNamespaceInstantiation,
+        ClassOrNamespace *enclosingTemplateClassInstantiation)
+{
+    QList<ClassOrNamespace *> processed;
+    while (enclosingTemplateClassInstantiation
+           && !processed.contains(enclosingTemplateClassInstantiation)) {
+        processed.append(enclosingTemplateClassInstantiation);
+        if (enclosingTemplateClassInstantiation == nestedClassOrNamespaceInstantiation)
+            return false;
+        enclosingTemplateClassInstantiation = enclosingTemplateClassInstantiation->parent();
+    }
+
+    return true;
+}
+
 namespace CPlusPlus {
 
 static inline bool compareName(const Name *name, const Name *other)
@@ -308,7 +324,7 @@ ClassOrNamespace *LookupContext::globalNamespace() const
 }
 
 ClassOrNamespace *LookupContext::lookupType(const Name *name, Scope *scope,
-                                            ClassOrNamespace* enclosingTemplateInstantiation,
+                                            ClassOrNamespace *enclosingBinding,
                                             QSet<const Declaration *> typedefsBeingResolved) const
 {
     if (! scope || ! name) {
@@ -351,7 +367,7 @@ ClassOrNamespace *LookupContext::lookupType(const Name *name, Scope *scope,
         }
         // try to find it in block (rare case but has priority before enclosing scope)
         // e.g.: void foo() { struct S {};  S s; }
-        if (ClassOrNamespace *b = bindings()->lookupType(scope, enclosingTemplateInstantiation)) {
+        if (ClassOrNamespace *b = bindings()->lookupType(scope, enclosingBinding)) {
             if (ClassOrNamespace *classOrNamespaceNestedInNestedBlock = b->lookupType(name, block))
                 return classOrNamespaceNestedInNestedBlock;
         }
@@ -360,13 +376,13 @@ ClassOrNamespace *LookupContext::lookupType(const Name *name, Scope *scope,
         if (ClassOrNamespace *found = lookupType(name, scope->enclosingScope()))
             return found;
 
-    } else if (ClassOrNamespace *b = bindings()->lookupType(scope, enclosingTemplateInstantiation)) {
+    } else if (ClassOrNamespace *b = bindings()->lookupType(scope, enclosingBinding)) {
         return b->lookupType(name);
     } else if (Class *scopeAsClass = scope->asClass()) {
         if (scopeAsClass->enclosingScope()->isBlock()) {
             if (ClassOrNamespace *b = lookupType(scopeAsClass->name(),
                                                  scopeAsClass->enclosingScope(),
-                                                 enclosingTemplateInstantiation,
+                                                 enclosingBinding,
                                                  typedefsBeingResolved)) {
                 return b->lookupType(name);
             }
@@ -377,9 +393,9 @@ ClassOrNamespace *LookupContext::lookupType(const Name *name, Scope *scope,
 }
 
 ClassOrNamespace *LookupContext::lookupType(Symbol *symbol,
-                                            ClassOrNamespace* enclosingTemplateInstantiation) const
+                                            ClassOrNamespace *enclosingBinding) const
 {
-    return bindings()->lookupType(symbol, enclosingTemplateInstantiation);
+    return bindings()->lookupType(symbol, enclosingBinding);
 }
 
 QList<LookupItem> LookupContext::lookup(const Name *name, Scope *scope) const
@@ -615,10 +631,15 @@ QList<LookupItem> ClassOrNamespace::lookup_helper(const Name *name, bool searchI
                 // a qualified name. For instance, a nested class which is forward declared
                 // in the class but defined outside it - we should capture both.
                 Symbol *match = 0;
+                QSet<ClassOrNamespace *> processed;
                 for (ClassOrNamespace *parentBinding = binding->parent();
                         parentBinding && !match;
-                        parentBinding = parentBinding->parent())
+                        parentBinding = parentBinding->parent()) {
+                    if (processed.contains(parentBinding))
+                        break;
+                    processed.insert(parentBinding);
                     match = parentBinding->lookupInScope(fullName);
+                }
 
                 if (match) {
                     LookupItem item;
@@ -632,8 +653,12 @@ QList<LookupItem> ClassOrNamespace::lookup_helper(const Name *name, bool searchI
         }
 
         QSet<ClassOrNamespace *> processed;
+        QSet<ClassOrNamespace *> processedOwnParents;
         ClassOrNamespace *binding = this;
         do {
+            if (processedOwnParents.contains(binding))
+                break;
+            processedOwnParents.insert(binding);
             lookup_helper(name, binding, &result, &processed, /*templateId = */ 0);
             binding = binding->_parent;
         } while (searchInEnclosingScope && binding);
@@ -680,8 +705,8 @@ void ClassOrNamespace::lookup_helper(const Name *name, ClassOrNamespace *binding
         foreach (ClassOrNamespace *u, binding->usings())
             lookup_helper(name, u, result, processed, binding->_templateId);
 
-        Anonymouses::const_iterator cit = binding->_anonymouses.begin();
-        Anonymouses::const_iterator citEnd = binding->_anonymouses.end();
+        Anonymouses::const_iterator cit = binding->_anonymouses.constBegin();
+        Anonymouses::const_iterator citEnd = binding->_anonymouses.constEnd();
         for (; cit != citEnd; ++cit) {
             const AnonymousNameId *anonymousNameId = cit.key();
             ClassOrNamespace *a = cit.value();
@@ -777,8 +802,8 @@ ClassOrNamespace *ClassOrNamespace::lookupType(const Name *name, Block *block)
 {
     flush();
 
-    QHash<Block *, ClassOrNamespace *>::const_iterator citBlock = _blocks.find(block);
-    if (citBlock != _blocks.end()) {
+    QHash<Block *, ClassOrNamespace *>::const_iterator citBlock = _blocks.constFind(block);
+    if (citBlock != _blocks.constEnd()) {
         ClassOrNamespace *nestedBlock = citBlock.value();
         QSet<ClassOrNamespace *> processed;
         if (ClassOrNamespace *foundInNestedBlock
@@ -790,7 +815,7 @@ ClassOrNamespace *ClassOrNamespace::lookupType(const Name *name, Block *block)
         }
     }
 
-    for (citBlock = _blocks.begin(); citBlock != _blocks.end(); ++citBlock) {
+    for (citBlock = _blocks.constBegin(); citBlock != _blocks.constEnd(); ++citBlock) {
         if (ClassOrNamespace *foundNestedBlock = citBlock.value()->lookupType(name, block))
             return foundNestedBlock;
     }
@@ -804,20 +829,36 @@ ClassOrNamespace *ClassOrNamespace::findType(const Name *name)
     return lookupType_helper(name, &processed, /*searchInEnclosingScope =*/ false, this);
 }
 
+ClassOrNamespace *ClassOrNamespace::findBlock_helper(Block *block,
+                                                     QSet<ClassOrNamespace *> *processed,
+                                                     bool searchInEnclosingScope)
+{
+    for (ClassOrNamespace *binding = this; binding; binding = binding->_parent) {
+        if (processed->contains(binding))
+            break;
+        processed->insert(binding);
+        binding->flush();
+        auto end = binding->_blocks.end();
+        auto citBlock = binding->_blocks.find(block);
+        if (citBlock != end)
+            return citBlock.value();
+
+        for (citBlock = binding->_blocks.begin(); citBlock != end; ++citBlock) {
+            if (ClassOrNamespace *foundNestedBlock =
+                    citBlock.value()->findBlock_helper(block, processed, false)) {
+                return foundNestedBlock;
+            }
+        }
+        if (!searchInEnclosingScope)
+            break;
+    }
+    return 0;
+}
+
 ClassOrNamespace *ClassOrNamespace::findBlock(Block *block)
 {
-    flush();
-
-    QHash<Block *, ClassOrNamespace *>::const_iterator citBlock = _blocks.find(block);
-    if (citBlock != _blocks.end())
-        return citBlock.value();
-
-    for (citBlock = _blocks.begin(); citBlock != _blocks.end(); ++citBlock) {
-        if (ClassOrNamespace *foundNestedBlock = citBlock.value()->findBlock(block))
-            return foundNestedBlock;
-    }
-
-    return 0;
+    QSet<ClassOrNamespace *> processed;
+    return findBlock_helper(block, &processed, true);
 }
 
 Symbol *ClassOrNamespace::lookupInScope(const QList<const Name *> &fullName)
@@ -910,8 +951,31 @@ ClassOrNamespace *ClassOrNamespace::lookupType_helper(const Name *name,
     return 0;
 }
 
-ClassOrNamespace *ClassOrNamespace::findSpecializationWithPointer(const TemplateNameId *templId,
-                                                         const TemplateNameIdTable &specializations)
+static ClassOrNamespace *findSpecializationWithMatchingTemplateArgument(const Name *argumentName,
+                                                                        ClassOrNamespace *reference)
+{
+    foreach (Symbol *s, reference->symbols()) {
+        if (Class *clazz = s->asClass()) {
+            if (Template *templateSpecialization = clazz->enclosingTemplate()) {
+                const unsigned argumentCountOfSpecialization
+                                    = templateSpecialization->templateParameterCount();
+                for (unsigned i = 0; i < argumentCountOfSpecialization; ++i) {
+                    if (TypenameArgument *tParam
+                            = templateSpecialization->templateParameterAt(i)->asTypenameArgument()) {
+                        if (const Name *name = tParam->name()) {
+                            if (compareName(name, argumentName))
+                                return reference;
+                        }
+                    }
+                }
+            }
+        }
+    }
+    return 0;
+}
+
+ClassOrNamespace *ClassOrNamespace::findSpecialization(const TemplateNameId *templId,
+                                                       const TemplateNameIdTable &specializations)
 {
     // we go through all specialization and try to find that one with template argument as pointer
     for (TemplateNameIdTable::const_iterator cit = specializations.begin();
@@ -937,6 +1001,21 @@ ClassOrNamespace *ClassOrNamespace::findSpecializationWithPointer(const Template
                         && specPointer->elementType().type()->isNamedType()) {
                     return cit->second;
                 }
+
+                ArrayType *specArray
+                        = specializationTemplateArgument.type()->asArrayType();
+                if (specArray && initializationTemplateArgument.type()->isArrayType()) {
+                    if (const NamedType *argumentNamedType
+                            = specArray->elementType().type()->asNamedType()) {
+                        if (const Name *argumentName = argumentNamedType->name()) {
+                            if (ClassOrNamespace *reference
+                                    = findSpecializationWithMatchingTemplateArgument(
+                                            argumentName, cit->second)) {
+                                return reference;
+                            }
+                        }
+                    }
+                }
             }
         }
     }
@@ -944,6 +1023,22 @@ ClassOrNamespace *ClassOrNamespace::findSpecializationWithPointer(const Template
     return 0;
 }
 
+ClassOrNamespace *ClassOrNamespace::findOrCreateNestedAnonymousType(
+        const AnonymousNameId *anonymousNameId)
+{
+    QHash<const AnonymousNameId *, ClassOrNamespace *>::const_iterator cit
+            = _anonymouses.constFind(anonymousNameId);
+    if (cit != _anonymouses.constEnd()) {
+        return cit.value();
+    } else {
+        ClassOrNamespace *newAnonymous = _factory->allocClassOrNamespace(this);
+        if (Q_UNLIKELY(debug))
+            newAnonymous->_name = anonymousNameId;
+        _anonymouses[anonymousNameId] = newAnonymous;
+        return newAnonymous;
+    }
+}
+
 ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespace *origin)
 {
     Q_ASSERT(name != 0);
@@ -951,20 +1046,8 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
 
     const_cast<ClassOrNamespace *>(this)->flush();
 
-    const AnonymousNameId *anonymousNameId = name->asAnonymousNameId();
-    if (anonymousNameId) {
-        QHash<const AnonymousNameId *, ClassOrNamespace *>::const_iterator cit
-                = _anonymouses.find(anonymousNameId);
-        if (cit != _anonymouses.end()) {
-            return cit.value();
-        } else {
-            ClassOrNamespace *newAnonymous = _factory->allocClassOrNamespace(this);
-            if (Q_UNLIKELY(debug))
-                newAnonymous->_name = anonymousNameId;
-            _anonymouses[anonymousNameId] = newAnonymous;
-            return newAnonymous;
-        }
-    }
+    if (const AnonymousNameId *anonymousNameId = name->asAnonymousNameId())
+        return findOrCreateNestedAnonymousType(anonymousNameId);
 
     Table::const_iterator it = _classOrNamespaces.find(name);
     if (it == _classOrNamespaces.end())
@@ -1000,8 +1083,8 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
             }
         } else {
             QMap<const TemplateNameId *, ClassOrNamespace *>::const_iterator citInstantiation
-                    = reference->_instantiations.find(templId);
-            if (citInstantiation != reference->_instantiations.end())
+                    = reference->_instantiations.constFind(templId);
+            if (citInstantiation != reference->_instantiations.constEnd())
                 return citInstantiation.value();
             TemplateNameId *nonConstTemplId = const_cast<TemplateNameId *>(templId);
             // make this instantiation looks like specialization which help to find
@@ -1014,7 +1097,7 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
                 reference = cit->second;
             } else {
                 ClassOrNamespace *specializationWithPointer
-                        = findSpecializationWithPointer(templId, specializations);
+                        = findSpecialization(templId, specializations);
                 if (specializationWithPointer)
                     reference = specializationWithPointer;
                 // TODO: find the best specialization(probably partial) for this instantiation
@@ -1085,6 +1168,10 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
 
             Subst subst(_control.data());
             if (_factory->expandTemplates()) {
+                const TemplateNameId *templSpecId
+                        = templateSpecialization->name()->asTemplateNameId();
+                const unsigned templSpecArgumentCount = templSpecId ?
+                            templSpecId->templateArgumentCount() : 0;
                 Clone cloner(_control.data());
                 for (unsigned i = 0; i < argumentCountOfSpecialization; ++i) {
                     const TypenameArgument *tParam
@@ -1099,6 +1186,12 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
                                 templId->templateArgumentAt(i):
                                 cloner.type(tParam->type(), &subst);
 
+                    if (i < templSpecArgumentCount
+                            && templSpecId->templateArgumentAt(i)->isPointerType()) {
+                        if (PointerType *pointerType = ty->asPointerType())
+                            ty = pointerType->elementType();
+                    }
+
                     subst.bind(cloner.name(name, &subst), ty);
                 }
 
@@ -1112,7 +1205,7 @@ ClassOrNamespace *ClassOrNamespace::nestedType(const Name *name, ClassOrNamespac
                         oo.showReturnTypes = true;
                         oo.showTemplateParameters = true;
                         qDebug() << "cloned" << oo(clone->type());
-                        if (Class *klass = s->asClass()) {
+                        if (Class *klass = clone->asClass()) {
                             const unsigned klassMemberCount = klass->memberCount();
                             for (unsigned i = 0; i < klassMemberCount; ++i){
                                 Symbol *klassMemberAsSymbol = klass->memberAt(i);
@@ -1273,6 +1366,10 @@ void ClassOrNamespace::NestedClassInstantiator::instantiate(ClassOrNamespace *en
             }
         }
 
+        if (isNestedInstantiationEnclosingTemplate(nestedClassOrNamespaceInstantiation,
+                                                   enclosingTemplateClass)) {
+            nestedClassOrNamespaceInstantiation->_parent = enclosingTemplateClassInstantiation;
+        }
         instantiate(nestedClassOrNamespace, nestedClassOrNamespaceInstantiation);
 
         enclosingTemplateClassInstantiation->_classOrNamespaces[nestedName] =
@@ -1424,19 +1521,20 @@ ClassOrNamespace *CreateBindings::globalNamespace() const
     return _globalNamespace;
 }
 
-ClassOrNamespace *CreateBindings::lookupType(Symbol *symbol, ClassOrNamespace* enclosingTemplateInstantiation)
+ClassOrNamespace *CreateBindings::lookupType(Symbol *symbol, ClassOrNamespace *enclosingBinding)
 {
     const QList<const Name *> path = LookupContext::path(symbol);
-    return lookupType(path, enclosingTemplateInstantiation);
+    return lookupType(path, enclosingBinding);
 }
 
-ClassOrNamespace *CreateBindings::lookupType(const QList<const Name *> &path, ClassOrNamespace* enclosingTemplateInstantiation)
+ClassOrNamespace *CreateBindings::lookupType(const QList<const Name *> &path,
+                                             ClassOrNamespace *enclosingBinding)
 {
     if (path.isEmpty())
         return _globalNamespace;
 
-    if (enclosingTemplateInstantiation) {
-        if (ClassOrNamespace *b = enclosingTemplateInstantiation->lookupType(path.last()))
+    if (enclosingBinding) {
+        if (ClassOrNamespace *b = enclosingBinding->lookupType(path.last()))
             return b;
     }
 
@@ -1607,11 +1705,17 @@ bool CreateBindings::visit(Declaration *decl)
 
 bool CreateBindings::visit(Function *function)
 {
+    ClassOrNamespace *previous = _currentClassOrNamespace;
+    ClassOrNamespace *binding = lookupType(function, previous);
+    if (!binding)
+        return false;
+    _currentClassOrNamespace = binding;
     for (unsigned i = 0, count = function->memberCount(); i < count; ++i) {
         Symbol *s = function->memberAt(i);
         if (Block *b = s->asBlock())
             visit(b);
     }
+    _currentClassOrNamespace = previous;
     return false;
 }
 
