@@ -38,6 +38,8 @@
 #include "qmljsqrcparser.h"
 #include "qmljsconstants.h"
 
+#include <extensionsystem/pluginmanager.h>
+
 #include <QDir>
 
 using namespace LanguageUtils;
@@ -213,6 +215,13 @@ Context::ImportsPerDocument LinkPrivate::linkImports()
     if (document) {
         // do it on document first, to make sure import errors are shown
         Imports *imports = new Imports(valueOwner);
+
+        // Add custom imports for the opened document
+        auto providers = ExtensionSystem::PluginManager::getObjects<CustomImportsProvider>();
+        foreach (const auto &provider, providers)
+            foreach (const auto &import, provider->imports(valueOwner, document.data()))
+                importCache.insert(ImportCacheKey(import.info), import);
+
         populateImportedTypes(imports, document);
         importsPerDocument.insert(document.data(), QSharedPointer<Imports>(imports));
     }
@@ -264,7 +273,7 @@ void LinkPrivate::populateImportedTypes(Imports *imports, Document::Ptr doc)
                 imports->setImportFailed();
                 if (info.ast()) {
                     error(doc, info.ast()->fileNameToken,
-                          Link::tr("file or directory not found"));
+                          Link::tr("File or directory not found."));
                 }
                 break;
             default:
@@ -427,10 +436,12 @@ Import LinkPrivate::importNonFile(Document::Ptr doc, const ImportInfo &importInf
         error(doc, locationFromRange(importInfo.ast()->firstSourceLocation(),
                                      importInfo.ast()->lastSourceLocation()),
               Link::tr(
-                  "QML module not found\n\n"
+                  "QML module not found.\n\n"
                   "Import paths:\n"
                   "%1\n\n"
                   "For qmake projects, use the QML_IMPORT_PATH variable to add import paths.\n"
+                  "For Qbs projects, declare and set a qmlImportPaths property in your product "
+                  "to add import paths.\n"
                   "For qmlproject projects, use the importPaths property to add import paths.").arg(
                   importPaths.join(QLatin1Char('\n'))));
     }
