@@ -33,6 +33,7 @@
 #include "qmljscodeformatter.h"
 #include "parser/qmljsast_p.h"
 #include "parser/qmljsastvisitor_p.h"
+#include "parser/qmljsengine_p.h"
 
 #include <QString>
 #include <QTextBlock>
@@ -406,15 +407,6 @@ protected:
         return false;
     }
 
-    bool firstOnLine()
-    {
-        foreach (const QChar &c, _line) {
-            if (!c.isSpace())
-                return false;
-        }
-        return true;
-    }
-
     void addPossibleSplit(qreal badness, int offset = 0)
     {
         Split s;
@@ -556,13 +548,14 @@ protected:
                 out(ast->typeToken);
             }
             out(" ");
-            out(ast->identifierToken);
             if (ast->statement) {
+                out(ast->identifierToken);
                 out(": ", ast->colonToken);
                 accept(ast->statement);
             } else if (ast->binding) {
-                out(": ", ast->colonToken);
                 accept(ast->binding);
+            } else {
+                out(ast->identifierToken);
             }
         } else { // signal
             out("signal ");
