@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -265,6 +265,41 @@ ULONG64 ExtensionContext::jsExecutionContext(ExtensionCommandContext &exc,
     return result;
 }
 
+ExtensionContext::CdbVersion ExtensionContext::cdbVersion()
+{
+    static CdbVersion version;
+    static bool first = true;
+    if (!first)
+        return version;
+    first = false;
+    startRecordingOutput();
+    const HRESULT hr = m_control->OutputVersionInformation(DEBUG_OUTCTL_ALL_CLIENTS);
+    if (FAILED(hr)) {
+        stopRecordingOutput();
+        return version;
+    }
+    const std::wstring &output = stopRecordingOutput();
+    const std::wstring &versionOutput = L"Microsoft (R) Windows Debugger Version";
+    auto majorPos = output.find(versionOutput);
+    if (majorPos == std::wstring::npos)
+        return version;
+    majorPos += versionOutput.length();
+    std::wstring::size_type minorPos;
+    std::wstring::size_type patchPos;
+    try {
+        version.major = std::stoi(output.substr(majorPos), &minorPos);
+        minorPos += majorPos + 1;
+        version.minor = std::stoi(output.substr(minorPos), &patchPos);
+        patchPos += minorPos + 1;
+        version.patch = std::stoi(output.substr(patchPos));
+    }
+    catch (...)
+    {
+        version.clear();
+    }
+    return version;
+}
+
 // Complete stop parameters with common parameters and report
 static inline ExtensionContext::StopReasonMap
     completeStopReasons(CIDebugClient *client, ExtensionContext::StopReasonMap stopReasons, ULONG ex)
@@ -448,14 +483,14 @@ bool ExtensionContext::call(const std::string &functionCall,
     HRESULT hr = m_control->Execute(DEBUG_OUTCTL_ALL_CLIENTS, call.c_str(), DEBUG_EXECUTE_ECHO);
     if (FAILED(hr)) {
         *errorMessage = msgDebugEngineComFailed("Execute", hr);
-        return 0;
+        return false;
     }
     // Execute in current thread. TODO: This must not crash, else we are in an inconsistent state
     // (need to call 'gh', etc.)
     hr = m_control->Execute(DEBUG_OUTCTL_ALL_CLIENTS, goCommandForCall(callFlags), DEBUG_EXECUTE_ECHO);
     if (FAILED(hr)) {
         *errorMessage = msgDebugEngineComFailed("Execute", hr);
-        return 0;
+        return false;
     }
     // Wait until finished
     startRecordingOutput();
