@@ -66,7 +66,6 @@
 #include <QDomDocument>
 
 namespace {
-    const QLatin1String AndroidDirName("android");
     const QLatin1String AndroidManifestName("AndroidManifest.xml");
     const QLatin1String AndroidLibsFileName("/res/values/libs.xml");
     const QLatin1String AndroidDefaultPropertiesName("project.properties");
@@ -281,7 +280,7 @@ QString AndroidManager::loadLocalJarsInitClasses(ProjectExplorer::Target *target
 
 QPair<int, int> AndroidManager::apiLevelRange()
 {
-    return qMakePair(9, 21);
+    return qMakePair(9, 23);
 }
 
 QString AndroidManager::androidNameForApiLevel(int x)
@@ -323,6 +322,10 @@ QString AndroidManager::androidNameForApiLevel(int x)
         return QLatin1String("Android 4.4W");
     case 21:
         return QLatin1String("Android 5.0");
+    case 22:
+        return QLatin1String("Android 5.1");
+    case 23:
+        return QLatin1String("Android 6.0");
     default:
         return tr("Unknown Android version. API Level: %1").arg(QString::number(x));
     }
@@ -491,16 +494,15 @@ void AndroidManager::cleanLibsOnDevice(ProjectExplorer::Target *target)
     const QString targetArch = AndroidManager::targetArch(target);
     if (targetArch.isEmpty())
         return;
-    int deviceAPILevel = AndroidManager::minimumSDK(target);
+    const int deviceAPILevel = AndroidManager::minimumSDK(target);
     AndroidDeviceInfo info = AndroidConfigurations::showDeviceDialog(target->project(), deviceAPILevel, targetArch, AndroidConfigurations::None);
-    if (info.serialNumber.isEmpty()) // aborted
+    if (info.serialNumber.isEmpty() && info.avdname.isEmpty()) // aborted
         return;
 
-    deviceAPILevel = info.sdk;
     QString deviceSerialNumber = info.serialNumber;
 
     if (info.type == AndroidDeviceInfo::Emulator) {
-        deviceSerialNumber = AndroidConfigurations::currentConfig().startAVD(deviceSerialNumber, deviceAPILevel, targetArch);
+        deviceSerialNumber = AndroidConfigurations::currentConfig().startAVD(info.avdname);
         if (deviceSerialNumber.isEmpty())
             Core::MessageManager::write(tr("Starting Android virtual device failed."));
     }
@@ -521,15 +523,14 @@ void AndroidManager::installQASIPackage(ProjectExplorer::Target *target, const Q
     const QString targetArch = AndroidManager::targetArch(target);
     if (targetArch.isEmpty())
         return;
-    int deviceAPILevel = AndroidManager::minimumSDK(target);
+    const int deviceAPILevel = AndroidManager::minimumSDK(target);
     AndroidDeviceInfo info = AndroidConfigurations::showDeviceDialog(target->project(), deviceAPILevel, targetArch, AndroidConfigurations::None);
-    if (info.serialNumber.isEmpty()) // aborted
+    if (info.serialNumber.isEmpty() && info.avdname.isEmpty()) // aborted
         return;
 
-    deviceAPILevel = info.sdk;
     QString deviceSerialNumber = info.serialNumber;
     if (info.type == AndroidDeviceInfo::Emulator) {
-        deviceSerialNumber = AndroidConfigurations::currentConfig().startAVD(deviceSerialNumber, deviceAPILevel, targetArch);
+        deviceSerialNumber = AndroidConfigurations::currentConfig().startAVD(info.avdname);
         if (deviceSerialNumber.isEmpty())
             Core::MessageManager::write(tr("Starting Android virtual device failed."));
     }
@@ -559,9 +560,9 @@ bool AndroidManager::checkKeystorePassword(const QString &keystorePath, const QS
               << keystorePasswd;
     QProcess proc;
     proc.start(AndroidConfigurations::currentConfig().keytoolPath().toString(), arguments);
-    if (!proc.waitForStarted(4000))
+    if (!proc.waitForStarted(10000))
         return false;
-    if (!proc.waitForFinished(4000)) {
+    if (!proc.waitForFinished(10000)) {
         proc.kill();
         proc.waitForFinished();
         return false;
@@ -588,9 +589,9 @@ bool AndroidManager::checkCertificatePassword(const QString &keystorePath, const
 
     QProcess proc;
     proc.start(AndroidConfigurations::currentConfig().keytoolPath().toString(), arguments);
-    if (!proc.waitForStarted(4000))
+    if (!proc.waitForStarted(10000))
         return false;
-    if (!proc.waitForFinished(4000)) {
+    if (!proc.waitForFinished(10000)) {
         proc.kill();
         proc.waitForFinished();
         return false;
@@ -699,9 +700,21 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target)
     AndroidBuildApkStep *buildApkStep
         = AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
 
-    if (!buildApkStep || !buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("gradlew")).exists())
+    if (!buildApkStep || !buildApkStep->useGradle() || !buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("gradlew")).exists())
         return false;
 
+    Utils::FileName wrapperProps(buildApkStep->androidPackageSourceDir());
+    wrapperProps.appendPath(QLatin1String("gradle/wrapper/gradle-wrapper.properties"));
+    if (wrapperProps.exists()) {
+        GradleProperties wrapperProperties = readGradleProperties(wrapperProps.toString());
+        QString distributionUrl = QString::fromLocal8Bit(wrapperProperties["distributionUrl"]);
+        QRegExp re(QLatin1String(".*services.gradle.org/distributions/gradle-2..*.zip"));
+        if (!re.exactMatch(distributionUrl)) {
+            wrapperProperties["distributionUrl"] = "https\\://services.gradle.org/distributions/gradle-2.2.1-all.zip";
+            mergeGradleProperties(wrapperProps.toString(), wrapperProperties);
+        }
+    }
+
     GradleProperties localProperties;
     localProperties["sdk.dir"] = AndroidConfigurations::currentConfig().sdkLocation().toString().toLocal8Bit();
     if (!mergeGradleProperties(buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("local.properties")).toString(), localProperties))
