@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -136,7 +136,7 @@ void BaseTextFind::resetIncrementalSearch()
 
 void BaseTextFind::clearHighlights()
 {
-    emit highlightAll(QString(), 0);
+    highlightAll(QString(), 0);
 }
 
 QString BaseTextFind::currentFindString() const
@@ -185,9 +185,9 @@ IFindSupport::Result BaseTextFind::findIncremental(const QString &txt, FindFlags
         showWrapIndicator(d->m_widget);
     }
     if (found)
-        emit highlightAll(txt, findFlags);
+        highlightAll(txt, findFlags);
     else
-        emit highlightAll(QString(), 0);
+        highlightAll(QString(), 0);
     return found ? Found : NotFound;
 }
 
@@ -392,8 +392,10 @@ void BaseTextFind::defineFindScope()
 {
     QTextCursor cursor = textCursor();
     if (cursor.hasSelection() && cursor.block() != cursor.document()->findBlock(cursor.anchor())) {
-        d->m_findScopeStart = QTextCursor(document()->docHandle(), qMax(0, cursor.selectionStart()));
-        d->m_findScopeEnd = QTextCursor(document()->docHandle(), cursor.selectionEnd());
+        d->m_findScopeStart = cursor;
+        d->m_findScopeStart.setPosition(qMax(0, cursor.selectionStart()));
+        d->m_findScopeEnd = cursor;
+        d->m_findScopeEnd.setPosition(cursor.selectionEnd());
         d->m_findScopeVerticalBlockSelectionFirstColumn = -1;
         d->m_findScopeVerticalBlockSelectionLastColumn = -1;
 
@@ -425,4 +427,9 @@ void BaseTextFind::clearFindScope()
                           d->m_findScopeVerticalBlockSelectionLastColumn);
 }
 
+void BaseTextFind::highlightAll(const QString &txt, FindFlags findFlags)
+{
+    emit highlightAllRequested(txt, findFlags);
+}
+
 } // namespace Core
