@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -34,15 +34,19 @@
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/coreplugin.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
+#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/findplaceholder.h>
 
 #include <extensionsystem/pluginmanager.h>
 
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
+#include <utils/theme/theme.h>
 
 #include <QDebug>
 #include <QSettings>
@@ -61,8 +65,8 @@ Q_DECLARE_METATYPE(Core::IFindFilter*)
 static const int MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT = 150;
 static const int FINDBUTTON_SPACER_WIDTH = 20;
 
-using namespace Core;
-using namespace Core::Internal;
+namespace Core {
+namespace Internal {
 
 FindToolBar::FindToolBar(FindPlugin *plugin, CurrentDocumentFind *currentDocumentFind)
     : m_plugin(plugin),
@@ -90,11 +94,14 @@ FindToolBar::FindToolBar(FindPlugin *plugin, CurrentDocumentFind *currentDocumen
     setSingleRow(false);
     m_ui.findEdit->setAttribute(Qt::WA_MacShowFocusRect, false);
     m_ui.replaceEdit->setAttribute(Qt::WA_MacShowFocusRect, false);
+    m_ui.replaceEdit->setFiltering(true);
 
-    connect(m_ui.findEdit, SIGNAL(editingFinished()), this, SLOT(invokeResetIncrementalSearch()));
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::editingFinished,
+            this, &FindToolBar::invokeResetIncrementalSearch);
 
-    m_ui.close->setIcon(QIcon(QLatin1String(Core::Constants::ICON_CLOSE_DOCUMENT)));
-    connect(m_ui.close, SIGNAL(clicked()), this, SLOT(hideAndResetFocus()));
+    setLightColoredIcon(false);
+    connect(m_ui.close, &QToolButton::clicked,
+            this, &FindToolBar::hideAndResetFocus);
 
     m_findCompleter->setModel(m_plugin->findCompletionModel());
     m_replaceCompleter->setModel(m_plugin->replaceCompletionModel());
@@ -104,166 +111,210 @@ FindToolBar::FindToolBar(FindPlugin *plugin, CurrentDocumentFind *currentDocumen
     m_ui.findEdit->setButtonVisible(Utils::FancyLineEdit::Left, true);
     m_ui.findEdit->setFiltering(true);
     m_ui.findEdit->setPlaceholderText(QString());
+    m_ui.findEdit->setOkColor(Utils::creatorTheme()->color(Utils::Theme::TextColorNormal));
+    m_ui.findEdit->setErrorColor(Utils::creatorTheme()->color(Utils::Theme::TextColorError));
     m_ui.findEdit->button(Utils::FancyLineEdit::Left)->setFocusPolicy(Qt::TabFocus);
+    m_ui.findEdit->setValidationFunction([this](Utils::FancyLineEdit *, QString *) {
+                                             return m_lastResult != IFindSupport::NotFound;
+                                         });
     m_ui.replaceEdit->setPlaceholderText(QString());
 
-    connect(m_ui.findEdit, SIGNAL(textChanged(QString)), this, SLOT(invokeFindIncremental()));
-    connect(m_ui.findEdit, SIGNAL(leftButtonClicked()),
-            this, SLOT(findEditButtonClicked()));
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::textChanged,
+            this, &FindToolBar::invokeFindIncremental);
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::leftButtonClicked,
+            this, &FindToolBar::findEditButtonClicked);
 
     // invoke{Find,Replace}Helper change the completion model. QueuedConnection is used to perform these
     // changes only after the completer's activated() signal is handled (QTCREATORBUG-8408)
-    connect(m_ui.findEdit, SIGNAL(returnPressed()), this, SLOT(invokeFindEnter()), Qt::QueuedConnection);
-    connect(m_ui.replaceEdit, SIGNAL(returnPressed()), this, SLOT(invokeReplaceEnter()), Qt::QueuedConnection);
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::returnPressed,
+            this, &FindToolBar::invokeFindEnter, Qt::QueuedConnection);
+    connect(m_ui.replaceEdit, &Utils::FancyLineEdit::returnPressed,
+            this, &FindToolBar::invokeReplaceEnter, Qt::QueuedConnection);
 
     QAction *shiftEnterAction = new QAction(m_ui.findEdit);
     shiftEnterAction->setShortcut(QKeySequence(tr("Shift+Enter")));
     shiftEnterAction->setShortcutContext(Qt::WidgetShortcut);
-    connect(shiftEnterAction, SIGNAL(triggered()), this, SLOT(invokeFindPrevious()));
+    connect(shiftEnterAction, &QAction::triggered,
+            this, &FindToolBar::invokeFindPrevious);
     m_ui.findEdit->addAction(shiftEnterAction);
     QAction *shiftReturnAction = new QAction(m_ui.findEdit);
     shiftReturnAction->setShortcut(QKeySequence(tr("Shift+Return")));
     shiftReturnAction->setShortcutContext(Qt::WidgetShortcut);
-    connect(shiftReturnAction, SIGNAL(triggered()), this, SLOT(invokeFindPrevious()));
+    connect(shiftReturnAction, &QAction::triggered,
+            this, &FindToolBar::invokeFindPrevious);
     m_ui.findEdit->addAction(shiftReturnAction);
 
     QAction *shiftEnterReplaceAction = new QAction(m_ui.replaceEdit);
     shiftEnterReplaceAction->setShortcut(QKeySequence(tr("Shift+Enter")));
     shiftEnterReplaceAction->setShortcutContext(Qt::WidgetShortcut);
-    connect(shiftEnterReplaceAction, SIGNAL(triggered()), this, SLOT(invokeReplacePrevious()));
+    connect(shiftEnterReplaceAction, &QAction::triggered,
+            this, &FindToolBar::invokeReplacePrevious);
     m_ui.replaceEdit->addAction(shiftEnterReplaceAction);
     QAction *shiftReturnReplaceAction = new QAction(m_ui.replaceEdit);
     shiftReturnReplaceAction->setShortcut(QKeySequence(tr("Shift+Return")));
     shiftReturnReplaceAction->setShortcutContext(Qt::WidgetShortcut);
-    connect(shiftReturnReplaceAction, SIGNAL(triggered()), this, SLOT(invokeReplacePrevious()));
+    connect(shiftReturnReplaceAction, &QAction::triggered,
+            this, &FindToolBar::invokeReplacePrevious);
     m_ui.replaceEdit->addAction(shiftReturnReplaceAction);
 
     // need to make sure QStringList is registered as metatype
     QMetaTypeId<QStringList>::qt_metatype_id();
 
     // register actions
-    Core::Context globalcontext(Core::Constants::C_GLOBAL);
-    Core::ActionContainer *mfind = Core::ActionManager::actionContainer(Constants::M_FIND);
-    Core::Command *cmd;
+    Context findcontext(Constants::C_FINDTOOLBAR);
+    ActionContainer *mfind = ActionManager::actionContainer(Constants::M_FIND);
+    Command *cmd;
 
-    m_ui.advancedButton->setDefaultAction(Core::ActionManager::command(Constants::ADVANCED_FIND)->action());
+    m_ui.advancedButton->setDefaultAction(ActionManager::command(Constants::ADVANCED_FIND)->action());
 
     m_goToCurrentFindAction = new QAction(this);
-    Core::ActionManager::registerAction(m_goToCurrentFindAction, Constants::S_RETURNTOEDITOR,
-                                        Context(Constants::C_FINDTOOLBAR));
-    connect(m_goToCurrentFindAction, SIGNAL(triggered()), this, SLOT(setFocusToCurrentFindSupport()));
+    ActionManager::registerAction(m_goToCurrentFindAction, Constants::S_RETURNTOEDITOR,
+                                        findcontext);
+    connect(m_goToCurrentFindAction, &QAction::triggered,
+            this, &FindToolBar::setFocusToCurrentFindSupport);
 
     QIcon icon = QIcon::fromTheme(QLatin1String("edit-find-replace"));
     m_findInDocumentAction = new QAction(icon, tr("Find/Replace"), this);
-    cmd = Core::ActionManager::registerAction(m_findInDocumentAction, Constants::FIND_IN_DOCUMENT, globalcontext);
+    cmd = ActionManager::registerAction(m_findInDocumentAction, Constants::FIND_IN_DOCUMENT);
     cmd->setDefaultKeySequence(QKeySequence::Find);
     mfind->addAction(cmd, Constants::G_FIND_CURRENTDOCUMENT);
     connect(m_findInDocumentAction, SIGNAL(triggered()), this, SLOT(openFind()));
 
+    // Pressing the find shortcut while focus is in the tool bar should not change the search text,
+    // so register a different find action for the tool bar
+    auto localFindAction = new QAction(this);
+    ActionManager::registerAction(localFindAction, Constants::FIND_IN_DOCUMENT, findcontext);
+    connect(localFindAction, &QAction::triggered, this, [this]() {
+        openFindToolBar(FindToolBar::OpenFlags(UpdateAll & ~UpdateFindText));
+    });
+
     if (QApplication::clipboard()->supportsFindBuffer()) {
         m_enterFindStringAction = new QAction(tr("Enter Find String"), this);
-        cmd = Core::ActionManager::registerAction(m_enterFindStringAction, "Find.EnterFindString", globalcontext);
+        cmd = ActionManager::registerAction(m_enterFindStringAction, "Find.EnterFindString");
         cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+E")));
         mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-        connect(m_enterFindStringAction, SIGNAL(triggered()), this, SLOT(putSelectionToFindClipboard()));
+        connect(m_enterFindStringAction, &QAction::triggered, this, &FindToolBar::putSelectionToFindClipboard);
         connect(QApplication::clipboard(), SIGNAL(findBufferChanged()), this, SLOT(updateFromFindClipboard()));
     }
 
     m_findNextAction = new QAction(tr("Find Next"), this);
-    cmd = Core::ActionManager::registerAction(m_findNextAction, Constants::FIND_NEXT, globalcontext);
+    cmd = ActionManager::registerAction(m_findNextAction, Constants::FIND_NEXT);
     cmd->setDefaultKeySequence(QKeySequence::FindNext);
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_findNextAction, SIGNAL(triggered()), this, SLOT(invokeFindNext()));
-    m_ui.findNextButton->setDefaultAction(cmd->action());
+    connect(m_findNextAction, &QAction::triggered, this, &FindToolBar::invokeGlobalFindNext);
+    m_localFindNextAction = new QAction(m_findNextAction->text(), this);
+    cmd = ActionManager::registerAction(m_localFindNextAction, Constants::FIND_NEXT, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localFindNextAction);
+    connect(m_localFindNextAction, &QAction::triggered, this, &FindToolBar::invokeFindNext);
+    m_ui.findNextButton->setDefaultAction(m_localFindNextAction);
 
     m_findPreviousAction = new QAction(tr("Find Previous"), this);
-    cmd = Core::ActionManager::registerAction(m_findPreviousAction, Constants::FIND_PREVIOUS, globalcontext);
+    cmd = ActionManager::registerAction(m_findPreviousAction, Constants::FIND_PREVIOUS);
     cmd->setDefaultKeySequence(QKeySequence::FindPrevious);
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_findPreviousAction, SIGNAL(triggered()), this, SLOT(invokeFindPrevious()));
-    m_ui.findPreviousButton->setDefaultAction(cmd->action());
+    connect(m_findPreviousAction, &QAction::triggered, this, &FindToolBar::invokeGlobalFindPrevious);
+    m_localFindPreviousAction = new QAction(m_findPreviousAction->text(), this);
+    cmd = ActionManager::registerAction(m_localFindPreviousAction, Constants::FIND_PREVIOUS, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localFindPreviousAction);
+    connect(m_localFindPreviousAction, &QAction::triggered, this, &FindToolBar::invokeFindPrevious);
+    m_ui.findPreviousButton->setDefaultAction(m_localFindPreviousAction);
 
     m_findNextSelectedAction = new QAction(tr("Find Next (Selected)"), this);
-    cmd = Core::ActionManager::registerAction(m_findNextSelectedAction, Constants::FIND_NEXT_SELECTED, globalcontext);
+    cmd = ActionManager::registerAction(m_findNextSelectedAction, Constants::FIND_NEXT_SELECTED);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+F3")));
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_findNextSelectedAction, SIGNAL(triggered()), this, SLOT(findNextSelected()));
+    connect(m_findNextSelectedAction, &QAction::triggered, this, &FindToolBar::findNextSelected);
 
     m_findPreviousSelectedAction = new QAction(tr("Find Previous (Selected)"), this);
-    cmd = Core::ActionManager::registerAction(m_findPreviousSelectedAction, Constants::FIND_PREV_SELECTED, globalcontext);
+    cmd = ActionManager::registerAction(m_findPreviousSelectedAction, Constants::FIND_PREV_SELECTED);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+F3")));
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_findPreviousSelectedAction, SIGNAL(triggered()), this, SLOT(findPreviousSelected()));
+    connect(m_findPreviousSelectedAction, &QAction::triggered, this, &FindToolBar::findPreviousSelected);
 
     m_replaceAction = new QAction(tr("Replace"), this);
-    cmd = Core::ActionManager::registerAction(m_replaceAction, Constants::REPLACE, globalcontext);
+    cmd = ActionManager::registerAction(m_replaceAction, Constants::REPLACE);
     cmd->setDefaultKeySequence(QKeySequence());
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_replaceAction, SIGNAL(triggered()), this, SLOT(invokeReplace()));
-    m_ui.replaceButton->setDefaultAction(cmd->action());
+    connect(m_replaceAction, &QAction::triggered, this, &FindToolBar::invokeGlobalReplace);
+    m_localReplaceAction = new QAction(m_replaceAction->text(), this);
+    cmd = ActionManager::registerAction(m_localReplaceAction, Constants::REPLACE, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localReplaceAction);
+    connect(m_localReplaceAction, &QAction::triggered, this, &FindToolBar::invokeReplace);
+    m_ui.replaceButton->setDefaultAction(m_localReplaceAction);
 
     m_replaceNextAction = new QAction(tr("Replace && Find"), this);
-    m_replaceNextAction->setIconText(tr("Replace && Find")); // work around bug in Qt that kills ampersands in tool button
-    cmd = Core::ActionManager::registerAction(m_replaceNextAction, Constants::REPLACE_NEXT, globalcontext);
+    cmd = ActionManager::registerAction(m_replaceNextAction, Constants::REPLACE_NEXT);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+=")));
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_replaceNextAction, SIGNAL(triggered()), this, SLOT(invokeReplaceNext()));
-    m_ui.replaceNextButton->setDefaultAction(cmd->action());
+    connect(m_replaceNextAction, &QAction::triggered, this, &FindToolBar::invokeGlobalReplaceNext);
+    m_localReplaceNextAction = new QAction(m_replaceNextAction->text(), this);
+    m_localReplaceNextAction->setIconText(m_replaceNextAction->text()); // Workaround QTBUG-23396
+    cmd = ActionManager::registerAction(m_localReplaceNextAction, Constants::REPLACE_NEXT, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localReplaceNextAction);
+    connect(m_localReplaceNextAction, &QAction::triggered, this, &FindToolBar::invokeReplaceNext);
+    m_ui.replaceNextButton->setDefaultAction(m_localReplaceNextAction);
 
     m_replacePreviousAction = new QAction(tr("Replace && Find Previous"), this);
-    cmd = Core::ActionManager::registerAction(m_replacePreviousAction, Constants::REPLACE_PREVIOUS, globalcontext);
-    // shortcut removed, clashes with Ctrl++ on many keyboard layouts
-    //cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+=")));
+    cmd = ActionManager::registerAction(m_replacePreviousAction, Constants::REPLACE_PREVIOUS);
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_replacePreviousAction, SIGNAL(triggered()), this, SLOT(invokeReplacePrevious()));
+    connect(m_replacePreviousAction, &QAction::triggered, this, &FindToolBar::invokeGlobalReplacePrevious);
+    m_localReplacePreviousAction = new QAction(m_replacePreviousAction->text(), this);
+    cmd = ActionManager::registerAction(m_localReplacePreviousAction, Constants::REPLACE_PREVIOUS, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localReplacePreviousAction);
+    connect(m_localReplacePreviousAction, &QAction::triggered, this, &FindToolBar::invokeReplacePrevious);
 
     m_replaceAllAction = new QAction(tr("Replace All"), this);
-    cmd = Core::ActionManager::registerAction(m_replaceAllAction, Constants::REPLACE_ALL, globalcontext);
+    cmd = ActionManager::registerAction(m_replaceAllAction, Constants::REPLACE_ALL);
     mfind->addAction(cmd, Constants::G_FIND_ACTIONS);
-    connect(m_replaceAllAction, SIGNAL(triggered()), this, SLOT(invokeReplaceAll()));
-    m_ui.replaceAllButton->setDefaultAction(cmd->action());
+    connect(m_replaceAllAction, &QAction::triggered, this, &FindToolBar::invokeGlobalReplaceAll);
+    m_localReplaceAllAction = new QAction(m_replaceAllAction->text(), this);
+    cmd = ActionManager::registerAction(m_localReplaceAllAction, Constants::REPLACE_ALL, findcontext);
+    cmd->augmentActionWithShortcutToolTip(m_localReplaceAllAction);
+    connect(m_localReplaceAllAction, &QAction::triggered, this, &FindToolBar::invokeReplaceAll);
+    m_ui.replaceAllButton->setDefaultAction(m_localReplaceAllAction);
 
     m_caseSensitiveAction = new QAction(tr("Case Sensitive"), this);
     m_caseSensitiveAction->setIcon(QIcon(QLatin1String(":/find/images/casesensitively.png")));
     m_caseSensitiveAction->setCheckable(true);
     m_caseSensitiveAction->setChecked(false);
-    cmd = Core::ActionManager::registerAction(m_caseSensitiveAction, Constants::CASE_SENSITIVE, globalcontext);
+    cmd = ActionManager::registerAction(m_caseSensitiveAction, Constants::CASE_SENSITIVE);
     mfind->addAction(cmd, Constants::G_FIND_FLAGS);
-    connect(m_caseSensitiveAction, SIGNAL(toggled(bool)), this, SLOT(setCaseSensitive(bool)));
+    connect(m_caseSensitiveAction, &QAction::toggled, this, &FindToolBar::setCaseSensitive);
 
     m_wholeWordAction = new QAction(tr("Whole Words Only"), this);
     m_wholeWordAction->setIcon(QIcon(QLatin1String(":/find/images/wholewords.png")));
     m_wholeWordAction->setCheckable(true);
     m_wholeWordAction->setChecked(false);
-    cmd = Core::ActionManager::registerAction(m_wholeWordAction, Constants::WHOLE_WORDS, globalcontext);
+    cmd = ActionManager::registerAction(m_wholeWordAction, Constants::WHOLE_WORDS);
     mfind->addAction(cmd, Constants::G_FIND_FLAGS);
-    connect(m_wholeWordAction, SIGNAL(toggled(bool)), this, SLOT(setWholeWord(bool)));
+    connect(m_wholeWordAction, &QAction::toggled, this, &FindToolBar::setWholeWord);
 
     m_regularExpressionAction = new QAction(tr("Use Regular Expressions"), this);
     m_regularExpressionAction->setIcon(QIcon(QLatin1String(":/find/images/regexp.png")));
     m_regularExpressionAction->setCheckable(true);
     m_regularExpressionAction->setChecked(false);
-    cmd = Core::ActionManager::registerAction(m_regularExpressionAction, Constants::REGULAR_EXPRESSIONS, globalcontext);
+    cmd = ActionManager::registerAction(m_regularExpressionAction, Constants::REGULAR_EXPRESSIONS);
     mfind->addAction(cmd, Constants::G_FIND_FLAGS);
-    connect(m_regularExpressionAction, SIGNAL(toggled(bool)), this, SLOT(setRegularExpressions(bool)));
+    connect(m_regularExpressionAction, &QAction::toggled, this, &FindToolBar::setRegularExpressions);
 
     m_preserveCaseAction = new QAction(tr("Preserve Case when Replacing"), this);
     m_preserveCaseAction->setIcon(QPixmap(QLatin1String(":/find/images/preservecase.png")));
     m_preserveCaseAction->setCheckable(true);
     m_preserveCaseAction->setChecked(false);
-    cmd = Core::ActionManager::registerAction(m_preserveCaseAction, Constants::PRESERVE_CASE, globalcontext);
+    cmd = ActionManager::registerAction(m_preserveCaseAction, Constants::PRESERVE_CASE);
     mfind->addAction(cmd, Constants::G_FIND_FLAGS);
-    connect(m_preserveCaseAction, SIGNAL(toggled(bool)), this, SLOT(setPreserveCase(bool)));
+    connect(m_preserveCaseAction, &QAction::toggled, this, &FindToolBar::setPreserveCase);
 
-    connect(m_currentDocumentFind, SIGNAL(candidateChanged()), this, SLOT(adaptToCandidate()));
-    connect(m_currentDocumentFind, SIGNAL(changed()), this, SLOT(updateToolBar()));
+    connect(m_currentDocumentFind, &CurrentDocumentFind::candidateChanged, this, &FindToolBar::adaptToCandidate);
+    connect(m_currentDocumentFind, &CurrentDocumentFind::changed, this, &FindToolBar::updateGlobalActions);
+    connect(m_currentDocumentFind, &CurrentDocumentFind::changed, this, &FindToolBar::updateToolBar);
+    updateGlobalActions();
     updateToolBar();
 
     m_findIncrementalTimer.setSingleShot(true);
     m_findStepTimer.setSingleShot(true);
-    connect(&m_findIncrementalTimer, SIGNAL(timeout()),
-            this, SLOT(invokeFindIncremental()));
-    connect(&m_findStepTimer, SIGNAL(timeout()), this, SLOT(invokeFindStep()));
+    connect(&m_findIncrementalTimer, &QTimer::timeout, this, &FindToolBar::invokeFindIncremental);
+    connect(&m_findStepTimer, &QTimer::timeout, this, &FindToolBar::invokeFindStep);
 }
 
 FindToolBar::~FindToolBar()
@@ -325,22 +376,30 @@ bool FindToolBar::eventFilter(QObject *obj, QEvent *event)
 
 void FindToolBar::adaptToCandidate()
 {
-    updateFindAction();
-    if (findToolBarPlaceHolder() == Core::FindToolBarPlaceHolder::getCurrent()) {
+    updateGlobalActions();
+    if (findToolBarPlaceHolder() == FindToolBarPlaceHolder::getCurrent()) {
         m_currentDocumentFind->acceptCandidate();
-        if (isVisible())
+        if (isVisible() && m_currentDocumentFind->isEnabled())
             m_currentDocumentFind->highlightAll(getFindText(), effectiveFindFlags());
     }
 }
 
-void FindToolBar::updateFindAction()
+void FindToolBar::updateGlobalActions()
 {
-    bool enabled = m_currentDocumentFind->candidateIsEnabled();
-    m_findInDocumentAction->setEnabled((toolBarHasFocus() && isEnabled()) || enabled);
+    IFindSupport *candidate = m_currentDocumentFind->candidate();
+    bool enabled = (candidate != 0);
+    bool replaceEnabled = enabled && candidate->supportsReplace();
+    m_findInDocumentAction->setEnabled(enabled || (toolBarHasFocus() && isEnabled()));
     m_findNextSelectedAction->setEnabled(enabled);
     m_findPreviousSelectedAction->setEnabled(enabled);
     if (QApplication::clipboard()->supportsFindBuffer())
         m_enterFindStringAction->setEnabled(enabled);
+    m_findNextAction->setEnabled(enabled);
+    m_findPreviousAction->setEnabled(enabled);
+    m_replaceAction->setEnabled(replaceEnabled);
+    m_replaceNextAction->setEnabled(replaceEnabled);
+    m_replacePreviousAction->setEnabled(replaceEnabled);
+    m_replaceAllAction->setEnabled(replaceEnabled);
 }
 
 void FindToolBar::updateToolBar()
@@ -349,13 +408,13 @@ void FindToolBar::updateToolBar()
     bool replaceEnabled = enabled && m_currentDocumentFind->supportsReplace();
     bool showAllControls = canShowAllControls(replaceEnabled);
 
-    m_findNextAction->setEnabled(enabled);
-    m_findPreviousAction->setEnabled(enabled);
+    m_localFindNextAction->setEnabled(enabled);
+    m_localFindPreviousAction->setEnabled(enabled);
 
-    m_replaceAction->setEnabled(replaceEnabled);
-    m_replaceNextAction->setEnabled(replaceEnabled);
-    m_replacePreviousAction->setEnabled(replaceEnabled);
-    m_replaceAllAction->setEnabled(replaceEnabled);
+    m_localReplaceAction->setEnabled(replaceEnabled);
+    m_localReplacePreviousAction->setEnabled(replaceEnabled);
+    m_localReplaceNextAction->setEnabled(replaceEnabled);
+    m_localReplaceAllAction->setEnabled(replaceEnabled);
 
     m_caseSensitiveAction->setEnabled(enabled);
     m_wholeWordAction->setEnabled(enabled);
@@ -418,12 +477,32 @@ void FindToolBar::invokeFindNext()
     invokeFindStep();
 }
 
+void FindToolBar::invokeGlobalFindNext()
+{
+    if (getFindText().isEmpty()) {
+        openFind();
+    } else {
+        acceptCandidateAndMoveToolBar();
+        invokeFindNext();
+    }
+}
+
 void FindToolBar::invokeFindPrevious()
 {
     setFindFlag(FindBackward, true);
     invokeFindStep();
 }
 
+void FindToolBar::invokeGlobalFindPrevious()
+{
+    if (getFindText().isEmpty()) {
+        openFind();
+    } else {
+        acceptCandidateAndMoveToolBar();
+        invokeFindPrevious();
+    }
+}
+
 QString FindToolBar::getFindText()
 {
     return m_ui.findEdit->text();
@@ -436,12 +515,14 @@ QString FindToolBar::getReplaceText()
 
 void FindToolBar::setFindText(const QString &text)
 {
-    disconnect(m_ui.findEdit, SIGNAL(textChanged(QString)), this, SLOT(invokeFindIncremental()));
+    disconnect(m_ui.findEdit, &Utils::FancyLineEdit::textChanged,
+               this, &FindToolBar::invokeFindIncremental);
     if (hasFindFlag(FindRegularExpression))
         m_ui.findEdit->setText(QRegExp::escape(text));
     else
         m_ui.findEdit->setText(text);
-    connect(m_ui.findEdit, SIGNAL(textChanged(QString)), this, SLOT(invokeFindIncremental()));
+    connect(m_ui.findEdit, &Utils::FancyLineEdit::textChanged,
+            this, &FindToolBar::invokeFindIncremental);
 }
 
 void FindToolBar::selectFindText()
@@ -457,6 +538,7 @@ void FindToolBar::invokeFindStep()
         m_plugin->updateFindCompletion(getFindText());
         IFindSupport::Result result =
             m_currentDocumentFind->findStep(getFindText(), effectiveFindFlags());
+        indicateSearchState(result);
         if (result == IFindSupport::NotYetFound)
             m_findStepTimer.start(50);
     }
@@ -470,6 +552,7 @@ void FindToolBar::invokeFindIncremental()
         QString text = getFindText();
         IFindSupport::Result result =
             m_currentDocumentFind->findIncremental(text, effectiveFindFlags());
+        indicateSearchState(result);
         if (result == IFindSupport::NotYetFound)
             m_findIncrementalTimer.start(50);
         if (text.isEmpty())
@@ -487,18 +570,36 @@ void FindToolBar::invokeReplace()
     }
 }
 
+void FindToolBar::invokeGlobalReplace()
+{
+    acceptCandidateAndMoveToolBar();
+    invokeReplace();
+}
+
 void FindToolBar::invokeReplaceNext()
 {
     setFindFlag(FindBackward, false);
     invokeReplaceStep();
 }
 
+void FindToolBar::invokeGlobalReplaceNext()
+{
+    acceptCandidateAndMoveToolBar();
+    invokeReplaceNext();
+}
+
 void FindToolBar::invokeReplacePrevious()
 {
     setFindFlag(FindBackward, true);
     invokeReplaceStep();
 }
 
+void FindToolBar::invokeGlobalReplacePrevious()
+{
+    acceptCandidateAndMoveToolBar();
+    invokeReplacePrevious();
+}
+
 void FindToolBar::invokeReplaceStep()
 {
     if (m_currentDocumentFind->isEnabled() && m_currentDocumentFind->supportsReplace()) {
@@ -516,6 +617,12 @@ void FindToolBar::invokeReplaceAll()
         m_currentDocumentFind->replaceAll(getFindText(), getReplaceText(), effectiveFindFlags());
 }
 
+void FindToolBar::invokeGlobalReplaceAll()
+{
+    acceptCandidateAndMoveToolBar();
+    invokeReplaceAll();
+}
+
 void FindToolBar::invokeResetIncrementalSearch()
 {
     m_findIncrementalTimer.stop();
@@ -530,7 +637,6 @@ void FindToolBar::putSelectionToFindClipboard()
     openFind(false);
     const QString text = m_currentDocumentFind->currentFindString();
     QApplication::clipboard()->setText(text, QClipboard::FindBuffer);
-    setFindText(text);
 }
 
 
@@ -566,12 +672,8 @@ void FindToolBar::updateIcons()
     bool regexp = effectiveFlags & FindRegularExpression;
     bool preserveCase = effectiveFlags & FindPreserveCase;
     if (!casesensitive && !wholewords && !regexp && !preserveCase) {
-        QPixmap pixmap(17, 17);
-        pixmap.fill(Qt::transparent);
-        QPainter painter(&pixmap);
-        const QPixmap mag = QPixmap(QLatin1String(Core::Constants::ICON_MAGNIFIER));
-        painter.drawPixmap(0, (pixmap.height() - mag.height()) / 2, mag);
-        m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left, pixmap);
+        m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left,
+                                       Utils::StyleHelper::dpiSpecificImageFile(QLatin1Literal(Constants::ICON_MAGNIFIER)));
     } else {
         m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left,
                                        IFindFilter::pixmapForFindFlags(effectiveFlags));
@@ -630,12 +732,12 @@ void FindToolBar::hideAndResetFocus()
     hide();
 }
 
-Core::FindToolBarPlaceHolder *FindToolBar::findToolBarPlaceHolder() const
+FindToolBarPlaceHolder *FindToolBar::findToolBarPlaceHolder() const
 {
-    QList<Core::FindToolBarPlaceHolder*> placeholders = ExtensionSystem::PluginManager::getObjects<Core::FindToolBarPlaceHolder>();
+    QList<FindToolBarPlaceHolder*> placeholders = ExtensionSystem::PluginManager::getObjects<FindToolBarPlaceHolder>();
     QWidget *candidate = QApplication::focusWidget();
     while (candidate) {
-        foreach (Core::FindToolBarPlaceHolder *ph, placeholders) {
+        foreach (FindToolBarPlaceHolder *ph, placeholders) {
             if (ph->owner() == candidate)
                 return ph;
         }
@@ -669,52 +771,85 @@ bool FindToolBar::canShowAllControls(bool replaceIsVisible) const
     return fullWidth - replaceFixedWidth >= MINIMUM_WIDTH_FOR_COMPLEX_LAYOUT;
 }
 
+/*!
+    Accepts the candidate find of the current focus widget (if any), and moves the tool bar
+    there, if it was visible before.
+*/
+void FindToolBar::acceptCandidateAndMoveToolBar()
+{
+    if (!m_currentDocumentFind->candidate())
+        return;
+    if (isVisible()) {
+        openFindToolBar(UpdateHighlight);
+    } else {
+        // Make sure we are really hidden, and not just because our parent was hidden.
+        // Otherwise when the tool bar gets visible again, it will be in a different widget than
+        // the current document find it acts on.
+        // Test case: Open find in navigation side bar, hide side bar, click into editor,
+        // trigger find next, show side bar
+        hide();
+        m_currentDocumentFind->acceptCandidate();
+    }
+}
+
+void FindToolBar::indicateSearchState(IFindSupport::Result searchState)
+{
+    m_lastResult = searchState;
+    m_ui.findEdit->validate();
+}
+
 void FindToolBar::openFind(bool focus)
 {
     setBackward(false);
-    openFindToolBar(focus);
+    OpenFlags flags = UpdateAll;
+    if (!focus) // remove focus flag
+        flags = flags & ~UpdateFocusAndSelect;
+    openFindToolBar(flags);
 }
 
-void FindToolBar::openFindToolBar(bool focus)
+void FindToolBar::openFindToolBar(OpenFlags flags)
 {
     installEventFilters();
-    Core::FindToolBarPlaceHolder *holder = findToolBarPlaceHolder();
+    FindToolBarPlaceHolder *holder = findToolBarPlaceHolder();
     if (!holder)
         return;
-    Core::FindToolBarPlaceHolder *previousHolder = Core::FindToolBarPlaceHolder::getCurrent();
+    FindToolBarPlaceHolder *previousHolder = FindToolBarPlaceHolder::getCurrent();
     if (previousHolder != holder) {
         if (previousHolder)
             previousHolder->setWidget(0);
         holder->setWidget(this);
-        Core::FindToolBarPlaceHolder::setCurrent(holder);
+        FindToolBarPlaceHolder::setCurrent(holder);
     }
     m_currentDocumentFind->acceptCandidate();
     holder->setVisible(true);
     setVisible(true);
-    // We do not want to change the text when we currently have the focus and user presses the
-    // find shortcut
-    if (!focus || !toolBarHasFocus()) {
+//     We do not want to change the text when we currently have the focus and user presses the
+//     find shortcut
+//    if (!focus || !toolBarHasFocus()) {
+    if (flags & UpdateFindText) {
         QString text = m_currentDocumentFind->currentFindString();
         if (!text.isEmpty())
             setFindText(text);
     }
-    if (focus)
+    if (flags & UpdateFocusAndSelect)
         setFocus();
-    m_currentDocumentFind->defineFindScope();
-    m_currentDocumentFind->highlightAll(getFindText(), effectiveFindFlags());
-    if (focus)
+    if (flags & UpdateFindScope)
+        m_currentDocumentFind->defineFindScope();
+    if (flags & UpdateHighlight)
+        m_currentDocumentFind->highlightAll(getFindText(), effectiveFindFlags());
+    if (flags & UpdateFocusAndSelect)
         selectFindText();
 }
 
 void FindToolBar::findNextSelected()
 {
-    openFind(false);
+    openFindToolBar(OpenFlags(UpdateAll & ~UpdateFocusAndSelect));
     invokeFindNext();
 }
 
 void FindToolBar::findPreviousSelected()
 {
-    openFind(false);
+    openFindToolBar(OpenFlags(UpdateAll & ~UpdateFocusAndSelect));
     invokeFindPrevious();
 }
 
@@ -743,7 +878,7 @@ void FindToolBar::resizeEvent(QResizeEvent *event)
 
 void FindToolBar::writeSettings()
 {
-    QSettings *settings = Core::ICore::settings();
+    QSettings *settings = ICore::settings();
     settings->beginGroup(QLatin1String("Find"));
     settings->beginGroup(QLatin1String("FindToolBar"));
     settings->setValue(QLatin1String("Backward"), QVariant((m_findFlags & FindBackward) != 0));
@@ -757,7 +892,7 @@ void FindToolBar::writeSettings()
 
 void FindToolBar::readSettings()
 {
-    QSettings *settings = Core::ICore::settings();
+    QSettings *settings = ICore::settings();
     settings->beginGroup(QLatin1String("Find"));
     settings->beginGroup(QLatin1String("FindToolBar"));
     FindFlags flags;
@@ -825,6 +960,12 @@ void FindToolBar::setBackward(bool backward)
     setFindFlag(FindBackward, backward);
 }
 
+void FindToolBar::setLightColoredIcon(bool lightColored)
+{
+    m_ui.close->setIcon(lightColored ? QIcon(QLatin1String(Constants::ICON_DARK_CLOSE))
+                                     : QIcon(QLatin1String(Constants::ICON_BUTTON_CLOSE)));
+}
+
 OptionsPopup::OptionsPopup(QWidget *parent)
     : QWidget(parent, Qt::Popup)
 {
@@ -885,8 +1026,11 @@ QCheckBox *OptionsPopup::createCheckboxForCommand(Id id)
     checkbox->setChecked(action->isChecked());
     checkbox->setEnabled(action->isEnabled());
     checkbox->installEventFilter(this); // enter key handling
-    QObject::connect(checkbox, SIGNAL(clicked(bool)), action, SLOT(setChecked(bool)));
-    QObject::connect(action, SIGNAL(changed()), this, SLOT(actionChanged()));
+    QObject::connect(checkbox, &QCheckBox::clicked, action, &QAction::setChecked);
+    QObject::connect(action, &QAction::changed, this, &OptionsPopup::actionChanged);
     m_checkboxMap.insert(action, checkbox);
     return checkbox;
 }
+
+} // namespace Internal
+} // namespace Core
