@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -59,11 +59,9 @@ public:
 
     static CppEditorPlugin *instance();
 
-    bool initialize(const QStringList &arguments, QString *errorMessage = 0);
-    void extensionsInitialized();
-    ShutdownFlag aboutToShutdown();
-
-    bool sortedOutline() const;
+    bool initialize(const QStringList &arguments, QString *errorMessage = 0) override;
+    void extensionsInitialized() override;
+    ShutdownFlag aboutToShutdown() override;
 
     CppQuickFixAssistProvider *quickFixProvider() const;
 
@@ -81,14 +79,15 @@ public slots:
     void renameSymbolUnderCursor();
     void switchDeclarationDefinition();
 
-    void setSortedOutline(bool sorted);
-
 private slots:
     void onTaskStarted(Core::Id type);
     void onAllTasksFinished(Core::Id type);
     void inspectCppCodeModel();
 
 #ifdef WITH_TESTS
+private:
+    QList<QObject *> createTestObjects() const override;
+
 private slots:
     // The following tests expect that no projects are loaded on start-up.
     void test_SwitchMethodDeclarationDefinition_data();
@@ -119,15 +118,14 @@ private slots:
     void test_FollowSymbolUnderCursor_virtualFunctionCall();
     void test_FollowSymbolUnderCursor_virtualFunctionCall_multipleDocuments();
 
-    void test_doxygen_comments_data();
-    void test_doxygen_comments();
-    void test_doxygen_comments_no_leading_asterisks_data();
-    void test_doxygen_comments_no_leading_asterisks();
-
     void test_quickfix_data();
     void test_quickfix();
 
     void test_quickfix_GenerateGetterSetter_basicGetterWithPrefixAndNamespaceToCpp();
+    void test_quickfix_GenerateGetterSetter_onlyGetter();
+    void test_quickfix_GenerateGetterSetter_onlySetter();
+    void test_quickfix_GenerateGetterSetter_offerGetterWhenSetterPresent();
+    void test_quickfix_GenerateGetterSetter_offerSetterWhenGetterPresent();
 
     void test_quickfix_ConvertQt4Connect_connectOutOfClass();
     void test_quickfix_ConvertQt4Connect_connectWithinClass_data();
@@ -154,34 +152,8 @@ private slots:
 
     void test_quickfix_InsertDeclFromDef();
 
-    void test_quickfix_AddIncludeForUndefinedIdentifier_onSimpleName();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_onNameOfQualifiedName();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_onBaseOfQualifiedName();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_onTemplateName();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_onTemplateNameInsideArguments();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_withForwardDeclaration();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_withForwardDeclaration2();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_withForwardHeader();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_ignoremoc();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_sortingTop();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_sortingMiddle();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_sortingBottom();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_appendToUnsorted();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_firstLocalIncludeAtFront();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_firstGlobalIncludeAtBack();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_preferGroupWithLongerMatchingPrefix();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_newGroupIfOnlyDifferentIncludeDirs();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedDirsSorted();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedDirsUnsorted();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedIncludeTypes1();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedIncludeTypes2();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedIncludeTypes3();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_mixedIncludeTypes4();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_noinclude();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_onlyIncludeGuard();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_veryFirstIncludeCppStyleCommentOnTop();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_veryFirstIncludeCStyleCommentOnTop();
-    void test_quickfix_AddIncludeForUndefinedIdentifier_inserting_checkQSomethingInQtIncludePaths();
+    void test_quickfix_AddIncludeForUndefinedIdentifier_data();
+    void test_quickfix_AddIncludeForUndefinedIdentifier();
     void test_quickfix_AddIncludeForUndefinedIdentifier_noDoubleQtHeaderInclude();
 
     void test_quickfix_MoveFuncDefOutside_MemberFuncToCpp();
@@ -200,6 +172,12 @@ private slots:
     void test_quickfix_MoveFuncDefOutside_respectWsInOperatorNames2();
     void test_quickfix_MoveFuncDefOutside_macroUses();
 
+    void test_quickfix_MoveAllFuncDefOutside_MemberFuncToCpp();
+    void test_quickfix_MoveAllFuncDefOutside_MemberFuncOutside();
+    void test_quickfix_MoveAllFuncDefOutside_DoNotTriggerOnBaseClass();
+    void test_quickfix_MoveAllFuncDefOutside_classWithBaseClass();
+    void test_quickfix_MoveAllFuncDefOutside_ignoreMacroCode();
+
     void test_quickfix_MoveFuncDefToDecl_MemberFunc();
     void test_quickfix_MoveFuncDefToDecl_MemberFuncOutside();
     void test_quickfix_MoveFuncDefToDecl_MemberFuncToCppNS();
@@ -210,9 +188,13 @@ private slots:
     void test_quickfix_MoveFuncDefToDecl_CtorWithInitialization();
     void test_quickfix_MoveFuncDefToDecl_structWithAssignedVariable();
     void test_quickfix_MoveFuncDefToDecl_macroUses();
+    void test_quickfix_MoveFuncDefToDecl_override();
 
     void test_quickfix_AssignToLocalVariable_templates();
 
+    void test_quickfix_ExtractFunction_data();
+    void test_quickfix_ExtractFunction();
+
     void test_quickfix_ExtractLiteralAsParameter_typeDeduction_data();
     void test_quickfix_ExtractLiteralAsParameter_typeDeduction();
     void test_quickfix_ExtractLiteralAsParameter_freeFunction_separateFiles();
@@ -231,8 +213,17 @@ private slots:
     void test_includehierarchy_data();
     void test_includehierarchy();
 
-    // The following tests depend on the projects that are loaded on startup
-    // and will be skipped in case no projects are loaded.
+    // The following tests operate on a project and require special invocation:
+    //
+    // Ensure that the project is properly configured for a given settings path:
+    //   $ ./qtcreator -settingspath /your/settings/path /path/to/project
+    //
+    // ...and that it builds, which might prevent blocking dialogs for not
+    // existing files (e.g. ui_*.h).
+    //
+    // Run a test:
+    //   $ export QTC_TEST_WAIT_FOR_LOADED_PROJECT=1
+    //   $ ./qtcreator -settingspath /your/settings/path -test CppEditor,test_openEachFile /path/to/project
     void test_openEachFile();
     void test_switchHeaderSourceOnEachFile();
     void test_moveTokenWiseThroughEveryFile();
@@ -247,12 +238,9 @@ private slots:
 
 private:
     Core::IEditor *createEditor(QWidget *parent);
-    void writeSettings();
-    void readSettings();
 
     static CppEditorPlugin *m_instance;
 
-    bool m_sortedOutline;
     QAction *m_renameSymbolUnderCursorAction;
     QAction *m_findUsagesAction;
     QAction *m_reparseExternallyChangedFiles;
