@@ -131,7 +131,11 @@ LineForNewIncludeDirective::LineForNewIncludeDirective(const QTextDocument *text
     , m_cppDocument(cppDocument)
     , m_includeStyle(includeStyle)
 {
-    const QList<Document::Include> includes = cppDocument->resolvedIncludes();
+    QList<Document::Include> includes
+        = cppDocument->resolvedIncludes() + cppDocument->unresolvedIncludes();
+    Utils::sort(includes, [](const Include &left, const Include &right) {
+        return left.line() < right.line();
+    });
 
     // Ignore *.moc includes if requested
     if (mocIncludeMode == IgnoreMocIncludes) {
@@ -143,13 +147,6 @@ LineForNewIncludeDirective::LineForNewIncludeDirective(const QTextDocument *text
         m_includes = includes;
     }
 
-    // TODO: Remove this filter loop once FastPreprocessor::sourceNeeded does not add
-    // extra includes anymore.
-    for (int i = m_includes.count() - 1; i >= 0; --i) {
-        if (!QFileInfo(m_includes.at(i).resolvedFileName()).isAbsolute())
-            m_includes.removeAt(i);
-    }
-
     // Detect include style
     if (m_includeStyle == AutoDetect) {
         unsigned timesIncludeStyleChanged = 0;
@@ -220,7 +217,7 @@ int LineForNewIncludeDirective::operator()(const QString &newIncludeFileName,
         *newLinesToAppend = false;
 
     const QString pureIncludeFileName = newIncludeFileName.mid(1, newIncludeFileName.length() - 2);
-    const CPlusPlus::Client::IncludeType newIncludeType =
+    const Client::IncludeType newIncludeType =
         newIncludeFileName.startsWith(QLatin1Char('"')) ? Client::IncludeLocal
                                                         : Client::IncludeGlobal;
 
@@ -328,11 +325,6 @@ QList<IncludeGroup> LineForNewIncludeDirective::getGroupsByIncludeType(
 /// includes will be modified!
 QList<IncludeGroup> IncludeGroup::detectIncludeGroupsByNewLines(QList<Document::Include> &includes)
 {
-    // Sort by line
-    Utils::sort(includes, [](const Include &left, const Include &right) {
-        return left.line() < right.line();
-    });
-
     // Create groups
     QList<IncludeGroup> result;
     unsigned lastLine = 0;
@@ -399,11 +391,11 @@ QList<IncludeGroup> IncludeGroup::detectIncludeGroupsByIncludeType(const QList<I
 {
     // Create sub groups
     QList<IncludeGroup> result;
-    CPlusPlus::Client::IncludeType lastIncludeType;
+    Client::IncludeType lastIncludeType = Client::IncludeLocal;
     QList<Include> currentIncludes;
     bool isFirst = true;
     foreach (const Include &include, includes) {
-        const CPlusPlus::Client::IncludeType currentIncludeType = include.type();
+        const Client::IncludeType currentIncludeType = include.type();
 
         // First include...
         if (isFirst) {
@@ -544,14 +536,8 @@ bool IncludeGroup::hasCommonIncludeDir() const
 using namespace Tests;
 using CppTools::Internal::CppToolsPlugin;
 
-static QList<Include> includesForSource(const QByteArray &source)
+static QList<Include> includesForSource(const QString &filePath)
 {
-    const QString fileName = TestIncludePaths::testFilePath();
-
-    FileWriterAndRemover scopedFile(fileName, source);
-    if (!scopedFile.writtenSuccessfully())
-        return QList<Include>();
-
     using namespace CppTools::Internal;
     CppModelManager *cmm = CppModelManager::instance();
     cmm->GC();
@@ -560,44 +546,18 @@ static QList<Include> includesForSource(const QByteArray &source)
                                     << ProjectPart::HeaderPath(
                                         TestIncludePaths::globalIncludePath(),
                                         ProjectPart::HeaderPath::IncludePath));
-    sourceProcessor->run(fileName);
+    sourceProcessor->run(filePath);
 
-    Document::Ptr document = cmm->document(fileName);
+    Document::Ptr document = cmm->document(filePath);
     return document->resolvedIncludes();
 }
 
 void CppToolsPlugin::test_includeGroups_detectIncludeGroupsByNewLines()
 {
-    // Source referencing those files
-    QByteArray source =
-        "#include \"header.h\"\n"
-        "\n"
-        "#include \"file.h\"\n"
-        "#include \"fileother.h\"\n"
-        "\n"
-        "#include <lib/fileother.h>\n"
-        "#include <lib/file.h>\n"
-        "\n"
-        "#include \"otherlib/file.h\"\n"
-        "#include \"otherlib/fileother.h\"\n"
-        "\n"
-        "#include \"utils/utils.h\"\n"
-        "\n"
-        "#include <QDebug>\n"
-        "#include <QDir>\n"
-        "#include <QString>\n"
-        "\n"
-        "#include <iostream>\n"
-        "#include <string>\n"
-        "#include <except>\n"
-        "\n"
-        "#include <iostream>\n"
-        "#include \"stuff\"\n"
-        "#include <except>\n"
-        "\n"
-        ;
+    const QString testFilePath = TestIncludePaths::testFilePath(
+                QLatin1String("test_main_detectIncludeGroupsByNewLines.cpp"));
 
-    QList<Include> includes = includesForSource(source);
+    QList<Include> includes = includesForSource(testFilePath);
     QCOMPARE(includes.size(), 17);
     QList<IncludeGroup> includeGroups
         = IncludeGroup::detectIncludeGroupsByNewLines(includes);
@@ -636,20 +596,10 @@ void CppToolsPlugin::test_includeGroups_detectIncludeGroupsByNewLines()
 
 void CppToolsPlugin::test_includeGroups_detectIncludeGroupsByIncludeDir()
 {
-    QByteArray source =
-        "#include \"file.h\"\n"
-        "#include \"fileother.h\"\n"
-        "#include <lib/file.h>\n"
-        "#include <lib/fileother.h>\n"
-        "#include \"otherlib/file.h\"\n"
-        "#include \"otherlib/fileother.h\"\n"
-        "#include <iostream>\n"
-        "#include <string>\n"
-        "#include <except>\n"
-        "\n"
-        ;
+    const QString testFilePath = TestIncludePaths::testFilePath(
+                QLatin1String("test_main_detectIncludeGroupsByIncludeDir.cpp"));
 
-    QList<Include> includes = includesForSource(source);
+    QList<Include> includes = includesForSource(testFilePath);
     QCOMPARE(includes.size(), 9);
     QList<IncludeGroup> includeGroups
         = IncludeGroup::detectIncludeGroupsByIncludeDir(includes);
@@ -670,20 +620,10 @@ void CppToolsPlugin::test_includeGroups_detectIncludeGroupsByIncludeDir()
 
 void CppToolsPlugin::test_includeGroups_detectIncludeGroupsByIncludeType()
 {
-    QByteArray source =
-        "#include \"file.h\"\n"
-        "#include \"fileother.h\"\n"
-        "#include <lib/file.h>\n"
-        "#include <lib/fileother.h>\n"
-        "#include \"otherlib/file.h\"\n"
-        "#include \"otherlib/fileother.h\"\n"
-        "#include <iostream>\n"
-        "#include <string>\n"
-        "#include <except>\n"
-        "\n"
-        ;
+    const QString testFilePath = TestIncludePaths::testFilePath(
+                QLatin1String("test_main_detectIncludeGroupsByIncludeType.cpp"));
 
-    QList<Include> includes = includesForSource(source);
+    QList<Include> includes = includesForSource(testFilePath);
     QCOMPARE(includes.size(), 9);
     QList<IncludeGroup> includeGroups
         = IncludeGroup::detectIncludeGroupsByIncludeDir(includes);
