@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -33,13 +33,17 @@
 
 #include "gitsettings.h"
 
+#include <coreplugin/id.h>
+
 #include <vcsbase/vcsbaseplugin.h>
 
 #include <QStringList>
 #include <QPointer>
-#include <QPair>
+#include <QKeySequence>
 #include <QVector>
 
+#include <functional>
+
 QT_BEGIN_NAMESPACE
 class QFile;
 class QAction;
@@ -48,11 +52,9 @@ QT_END_NAMESPACE
 
 namespace Core {
 class IEditor;
-class IEditorFactory;
 class Command;
 class CommandLocator;
 class Context;
-class ActionManager;
 class ActionContainer;
 }
 namespace Utils { class ParameterAction; }
@@ -72,9 +74,6 @@ class RemoteDialog;
 
 typedef void (GitClient::*GitClientMemberFunc)(const QString &);
 
-typedef QPair<QAction *, Core::Command* > ActionCommandPair;
-typedef QPair<Utils::ParameterAction *, Core::Command* > ParameterActionCommandPair;
-
 class GitPlugin : public VcsBase::VcsBasePlugin
 {
     Q_OBJECT
@@ -86,14 +85,11 @@ public:
 
     static GitPlugin *instance();
 
-    bool initialize(const QStringList &arguments, QString *errorMessage);
+    bool initialize(const QStringList &arguments, QString *errorMessage) override;
 
     GitVersionControl *gitVersionControl() const;
 
-    const GitSettings &settings() const;
-    void setSettings(const GitSettings &s);
-
-    GitClient *gitClient() const;
+    GitClient *client() const;
     Gerrit::Internal::GerritPlugin *gerritPlugin() const;
 
 public slots:
@@ -116,7 +112,7 @@ private slots:
     void undoUnstagedFileChanges();
     void resetRepository();
     void startRebase();
-    void startChangeRelatedAction();
+    void startChangeRelatedAction(const Core::Id &id);
     void stageFile();
     void unstageFile();
     void gitkForCurrentFile();
@@ -127,12 +123,13 @@ private slots:
     void updateSubmodules();
     void applyCurrentFilePatch();
     void promptApplyPatch();
-    void gitClientMemberFuncRepositoryAction();
 
     void startAmendCommit();
     void startFixupCommit();
-    void stash();
+    void stash(bool unstagedOnly = false);
+    void stashUnstaged();
     void stashSnapshot();
+    void stashPop();
     void branchList();
     void remoteList();
     void stashList();
@@ -150,48 +147,53 @@ private slots:
     void testDiffFileResolving_data();
     void testDiffFileResolving();
     void testLogResolving();
-    void testCloneWizard_directoryFromRepository();
-    void testCloneWizard_directoryFromRepository_data();
 #endif
 protected:
-    void updateActions(VcsBase::VcsBasePlugin::ActionState);
-    bool submitEditorAboutToClose();
+    void updateActions(VcsBase::VcsBasePlugin::ActionState) override;
+    bool submitEditorAboutToClose() override;
 
 private:
-    inline ParameterActionCommandPair
-            createParameterAction(Core::ActionContainer *ac,
-                                  const QString &defaultText, const QString &parameterText,
-                                  Core::Id id, const Core::Context &context, bool addToLocator);
+    Utils::ParameterAction *createParameterAction(Core::ActionContainer *ac,
+                                                  const QString &defaultText, const QString &parameterText,
+                                                  Core::Id id, const Core::Context &context, bool addToLocator,
+                                                  const QKeySequence &keys = QKeySequence());
 
-    inline ParameterActionCommandPair
-            createFileAction(Core::ActionContainer *ac,
-                             const QString &defaultText, const QString &parameterText,
-                             Core::Id id, const Core::Context &context, bool addToLocator,
-                             const char *pluginSlot);
+    QAction *createFileAction(Core::ActionContainer *ac,
+                              const QString &defaultText, const QString &parameterText,
+                              Core::Id id, const Core::Context &context, bool addToLocator,
+                              const char *pluginSlot,
+                              const QKeySequence &keys = QKeySequence());
 
-    inline ParameterActionCommandPair
-            createProjectAction(Core::ActionContainer *ac,
-                                const QString &defaultText, const QString &parameterText,
-                                Core::Id id, const Core::Context &context, bool addToLocator);
+    QAction *createProjectAction(Core::ActionContainer *ac,
+                                 const QString &defaultText, const QString &parameterText,
+                                 Core::Id id, const Core::Context &context, bool addToLocator,
+                                 const QKeySequence &keys = QKeySequence());
 
-    inline ParameterActionCommandPair
-                createProjectAction(Core::ActionContainer *ac,
-                                    const QString &defaultText, const QString &parameterText,
-                                    Core::Id id, const Core::Context &context, bool addToLocator,
-                                    const char *pluginSlot);
+    QAction *createProjectAction(Core::ActionContainer *ac,
+                                 const QString &defaultText, const QString &parameterText,
+                                 Core::Id id, const Core::Context &context, bool addToLocator,
+                                 const char *pluginSlot, const QKeySequence &keys = QKeySequence());
 
 
-    inline ActionCommandPair createRepositoryAction(Core::ActionContainer *ac,
-                                           const QString &text, Core::Id id,
-                                           const Core::Context &context, bool addToLocator);
-    inline ActionCommandPair createRepositoryAction(Core::ActionContainer *ac,
-                                           const QString &text, Core::Id id,
-                                           const Core::Context &context,
-                                           bool addToLocator, const char *pluginSlot);
-    inline ActionCommandPair createRepositoryAction(Core::ActionContainer *ac,
-                                           const QString &text, Core::Id id,
-                                           const Core::Context &context,
-                                           bool addToLocator, GitClientMemberFunc);
+    QAction *createRepositoryAction(Core::ActionContainer *ac,
+                                    const QString &text, Core::Id id,
+                                    const Core::Context &context, bool addToLocator,
+                                    const QKeySequence &keys = QKeySequence());
+    QAction *createRepositoryAction(Core::ActionContainer *ac,
+                                    const QString &text, Core::Id id,
+                                    const Core::Context &context,
+                                    bool addToLocator, const std::function<void()> &callback,
+                                    const QKeySequence &keys = QKeySequence());
+    QAction *createChangeRelatedRepositoryAction(Core::ActionContainer *ac,
+                                                 const QString &text, Core::Id id,
+                                                 const Core::Context &context,
+                                                 bool addToLocator, const std::function<void(Core::Id)> &callback,
+                                                 const QKeySequence &keys = QKeySequence());
+    QAction *createRepositoryAction(Core::ActionContainer *ac,
+                                    const QString &text, Core::Id id,
+                                    const Core::Context &context,
+                                    bool addToLocator, GitClientMemberFunc,
+                                    const QKeySequence &keys = QKeySequence());
 
     void updateRepositoryBrowserAction();
     bool isCommitEditorOpen() const;
@@ -202,7 +204,6 @@ private:
     void startCommit(CommitType commitType);
     void updateVersionWarning();
 
-    static GitPlugin *m_instance;
     Core::CommandLocator *m_commandLocator;
 
     QAction *m_submitCurrentAction;
@@ -222,6 +223,7 @@ private:
     QAction *m_continueRevertAction;
     QAction *m_fixupCommitAction;
     QAction *m_interactiveRebaseAction;
+    QAction *m_createRepositryAction;
 
     QVector<Utils::ParameterAction *> m_fileActions;
     QVector<Utils::ParameterAction *> m_projectActions;
@@ -229,18 +231,16 @@ private:
     Utils::ParameterAction *m_applyCurrentFilePatchAction;
     Gerrit::Internal::GerritPlugin *m_gerritPlugin;
 
-    GitClient                   *m_gitClient;
-    QPointer<StashDialog>       m_stashDialog;
-    QPointer<BranchDialog>      m_branchDialog;
-    QPointer<RemoteDialog>      m_remoteDialog;
-    QString                     m_submitRepository;
-    QString                     m_commitMessageFileName;
-    bool                        m_submitActionTriggered;
-
-    GitSettings m_settings;
+    GitClient *m_gitClient;
+    QPointer<StashDialog> m_stashDialog;
+    QPointer<BranchDialog> m_branchDialog;
+    QPointer<RemoteDialog> m_remoteDialog;
+    QString m_submitRepository;
+    QString m_commitMessageFileName;
+    bool m_submitActionTriggered;
 };
 
-} // namespace Git
 } // namespace Internal
+} // namespace Git
 
 #endif // GITPLUGIN_H
