@@ -47,7 +47,10 @@ namespace ProjectExplorer {
 const char lastKitKey[] = "LastSelectedKit";
 
 KitChooser::KitChooser(QWidget *parent) :
-    QWidget(parent)
+    QWidget(parent),
+    m_kitMatcher([](const Kit *k) {
+        return k->isValid();
+    })
 {
     m_chooser = new QComboBox(this);
     m_chooser->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
@@ -67,8 +70,7 @@ KitChooser::KitChooser(QWidget *parent) :
 
 void KitChooser::onManageButtonClicked()
 {
-    Core::ICore::showOptionsDialog(Constants::PROJECTEXPLORER_SETTINGS_CATEGORY,
-           Constants::KITS_SETTINGS_PAGE_ID, this);
+    Core::ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID, this);
 }
 
 void KitChooser::onCurrentIndexChanged(int index)
@@ -80,11 +82,6 @@ void KitChooser::onCurrentIndexChanged(int index)
     emit currentIndexChanged(index);
 }
 
-bool KitChooser::kitMatches(const Kit *k) const
-{
-    return k->isValid();
-}
-
 QString KitChooser::kitText(const Kit *k) const
 {
     return k->displayName();
@@ -98,8 +95,8 @@ QString KitChooser::kitToolTip(Kit *k) const
 void KitChooser::populate()
 {
     m_chooser->clear();
-    foreach (Kit *kit, KitManager::sortedKits()) {
-        if (kitMatches(kit)) {
+    foreach (Kit *kit, KitManager::sortKits(KitManager::kits())) {
+        if (m_kitMatcher(kit)) {
             m_chooser->addItem(kitText(kit), qVariantFromValue(kit->id()));
             m_chooser->setItemData(m_chooser->count() - 1, kitToolTip(kit), Qt::ToolTipRole);
         }
@@ -141,6 +138,12 @@ Core::Id KitChooser::currentKitId() const
     return kit ? kit->id() : Core::Id();
 }
 
+void KitChooser::setKitMatcher(const KitChooser::KitMatcher &matcher)
+{
+    m_kitMatcher = matcher;
+    populate();
+}
+
 Kit *KitChooser::kitAt(int index) const
 {
     Core::Id id = qvariant_cast<Core::Id>(m_chooser->itemData(index));
