@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -40,6 +40,7 @@
 #include "deployconfiguration.h"
 #include "project.h"
 #include "runconfiguration.h"
+#include "session.h"
 
 #include <limits>
 #include <coreplugin/coreconstants.h>
@@ -48,6 +49,7 @@
 #include <extensionsystem/pluginmanager.h>
 #include <projectexplorer/projectexplorer.h>
 #include <utils/qtcassert.h>
+#include <utils/algorithm.h>
 
 #include <QDebug>
 #include <QIcon>
@@ -137,6 +139,22 @@ Target::Target(Project *project, Kit *k) :
             this, SLOT(handleKitUpdates(ProjectExplorer::Kit*)));
     connect(km, SIGNAL(kitRemoved(ProjectExplorer::Kit*)),
             this, SLOT(handleKitRemoval(ProjectExplorer::Kit*)));
+
+    Utils::MacroExpander *expander = macroExpander();
+    expander->setDisplayName(tr("Target Settings"));
+    expander->setAccumulating(true);
+
+    expander->registerSubProvider([this] { return kit()->macroExpander(); });
+
+    expander->registerVariable("sourceDir", tr("Source directory"),
+            [project] { return project->projectDirectory().toUserOutput(); });
+
+    // Legacy support.
+    expander->registerVariable(Constants::VAR_CURRENTPROJECT_NAME,
+            QCoreApplication::translate("ProjectExplorer", "Name of current project"),
+            [project] { return project->displayName(); },
+            false);
+
 }
 
 Target::~Target()
@@ -219,9 +237,7 @@ void Target::addBuildConfiguration(BuildConfiguration *configuration)
 
     // Check that we don't have a configuration with the same displayName
     QString configurationDisplayName = configuration->displayName();
-    QStringList displayNames;
-    foreach (const BuildConfiguration *bc, d->m_buildConfigurations)
-        displayNames << bc->displayName();
+    QStringList displayNames = Utils::transform(d->m_buildConfigurations, &BuildConfiguration::displayName);
     configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
     if (configurationDisplayName != configuration->displayName()) {
         if (configuration->usesDefaultDisplayName())
@@ -261,9 +277,9 @@ bool Target::removeBuildConfiguration(BuildConfiguration *configuration)
 
     if (activeBuildConfiguration() == configuration) {
         if (d->m_buildConfigurations.isEmpty())
-            setActiveBuildConfiguration(0);
+            SessionManager::setActiveBuildConfiguration(this, 0, SetActive::Cascade);
         else
-            setActiveBuildConfiguration(d->m_buildConfigurations.at(0));
+            SessionManager::setActiveBuildConfiguration(this, d->m_buildConfigurations.at(0), SetActive::Cascade);
     }
 
     delete configuration;
@@ -303,9 +319,7 @@ void Target::addDeployConfiguration(DeployConfiguration *dc)
 
     // Check that we don't have a configuration with the same displayName
     QString configurationDisplayName = dc->displayName();
-    QStringList displayNames;
-    foreach (const DeployConfiguration *current, d->m_deployConfigurations)
-        displayNames << current->displayName();
+    QStringList displayNames = Utils::transform(d->m_deployConfigurations, &DeployConfiguration::displayName);
     configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
     dc->setDisplayName(configurationDisplayName);
 
@@ -336,9 +350,10 @@ bool Target::removeDeployConfiguration(DeployConfiguration *dc)
 
     if (activeDeployConfiguration() == dc) {
         if (d->m_deployConfigurations.isEmpty())
-            setActiveDeployConfiguration(0);
+            SessionManager::setActiveDeployConfiguration(this, 0, SetActive::Cascade);
         else
-            setActiveDeployConfiguration(d->m_deployConfigurations.at(0));
+            SessionManager::setActiveDeployConfiguration(this, d->m_deployConfigurations.at(0),
+                                                         SetActive::Cascade);
     }
 
     delete dc;
@@ -406,9 +421,7 @@ void Target::addRunConfiguration(RunConfiguration* runConfiguration)
 
     // Check that we don't have a configuration with the same displayName
     QString configurationDisplayName = runConfiguration->displayName();
-    QStringList displayNames;
-    foreach (const RunConfiguration *rc, d->m_runConfigurations)
-        displayNames << rc->displayName();
+    QStringList displayNames = Utils::transform(d->m_runConfigurations, &RunConfiguration::displayName);
     configurationDisplayName = Project::makeUnique(configurationDisplayName, displayNames);
     runConfiguration->setDisplayName(configurationDisplayName);
 
@@ -625,13 +638,10 @@ void Target::updateDefaultRunConfigurations()
 
     // Create new RCs and put them into newConfigured/newUnconfigured
     foreach (Core::Id id, autoCreateFactoryIds) {
-        IRunConfigurationFactory *factory = 0;
-        foreach (IRunConfigurationFactory *i, rcFactories) {
-            if (i->canCreate(this, id)) {
-                factory = i;
-                break;
-            }
-        }
+        IRunConfigurationFactory *factory = Utils::findOrDefault(rcFactories,
+                                                          [this, id] (IRunConfigurationFactory *i) {
+                                                                return i->canCreate(this, id);
+                                                          });
         if (!factory)
             continue;
 
@@ -688,12 +698,8 @@ void Target::updateDefaultRunConfigurations()
             RunConfiguration *selected = newConfigured.at(0);
             // Try to find a runconfiguration that matches the project name. That is a good
             // candidate for something to run initially.
-            foreach (RunConfiguration *rc, newConfigured) {
-                if (rc->displayName() == project()->displayName()) {
-                    selected = rc;
-                    break;
-                }
-            }
+            selected = Utils::findOr(newConfigured, selected,
+                                     Utils::equal(&RunConfiguration::displayName, project()->displayName()));
             setActiveRunConfiguration(selected);
         } else if (!newUnconfigured.isEmpty()){
             setActiveRunConfiguration(newUnconfigured.at(0));
