@@ -35,12 +35,15 @@
 #include "qbsproject.h"
 
 #include <coreplugin/coreconstants.h>
+#include <projectexplorer/buildmanager.h>
 #include <projectexplorer/buildstep.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/deployconfiguration.h>
 #include <projectexplorer/localapplicationruncontrol.h>
 #include <projectexplorer/localenvironmentaspect.h>
+#include <projectexplorer/runconfigurationaspects.h>
 #include <projectexplorer/target.h>
+#include <projectexplorer/runconfigurationaspects.h>
 #include <utils/qtcprocess.h>
 #include <utils/pathchooser.h>
 #include <utils/detailswidget.h>
@@ -68,9 +71,6 @@ namespace QbsProjectManager {
 namespace Internal {
 
 const char QBS_RC_PREFIX[] = "Qbs.RunConfiguration:";
-const char COMMAND_LINE_ARGUMENTS_KEY[] = "Qbs.RunConfiguration.CommandLineArguments";
-const char USE_TERMINAL_KEY[] = "Qbs.RunConfiguration.UseTerminal";
-const char USER_WORKING_DIRECTORY_KEY[] = "Qbs.RunConfiguration.UserWorkingDirectory";
 
 static QString rcNameSeparator() { return QLatin1String("---Qbs.RC.NameSeparator---"); }
 
@@ -113,14 +113,16 @@ const qbs::ProductData findProduct(const qbs::ProjectData &pro, const QString &u
 QbsRunConfiguration::QbsRunConfiguration(Target *parent, Core::Id id) :
     LocalApplicationRunConfiguration(parent, id),
     m_uniqueProductName(uniqueProductNameFromId(id)),
-    m_runMode(ApplicationLauncher::Gui),
     m_currentInstallStep(0),
     m_currentBuildStepList(0)
 {
     addExtraAspect(new LocalEnvironmentAspect(this));
+    addExtraAspect(new ArgumentsAspect(this, QStringLiteral("Qbs.RunConfiguration.CommandLineArguments")));
+    addExtraAspect(new WorkingDirectoryAspect(this, QStringLiteral("Qbs.RunConfiguration.WorkingDirectory")));
 
-    m_runModeForced = false;
-    m_runMode = isConsoleApplication() ? ApplicationLauncher::Console : ApplicationLauncher::Gui;
+    addExtraAspect(new TerminalAspect(this,
+                                      QStringLiteral("Qbs.RunConfiguration.UseTerminal"),
+                                      isConsoleApplication()));
 
     ctor();
 }
@@ -128,10 +130,6 @@ QbsRunConfiguration::QbsRunConfiguration(Target *parent, Core::Id id) :
 QbsRunConfiguration::QbsRunConfiguration(Target *parent, QbsRunConfiguration *source) :
     LocalApplicationRunConfiguration(parent, source),
     m_uniqueProductName(source->m_uniqueProductName),
-    m_commandLineArguments(source->m_commandLineArguments),
-    m_runMode(source->m_runMode),
-    m_runModeForced(source->m_runModeForced),
-    m_userWorkingDirectory(source->m_userWorkingDirectory),
     m_currentInstallStep(0), // no need to copy this, we will get if from the DC anyway.
     m_currentBuildStepList(0) // ditto
 {
@@ -162,11 +160,17 @@ void QbsRunConfiguration::ctor()
     QbsProject *project = static_cast<QbsProject *>(target()->project());
     connect(project, &QbsProject::projectParsingStarted, this, &RunConfiguration::enabledChanged);
     connect(project, &QbsProject::projectParsingDone, this, [this](bool success) {
-        if (success && !m_runModeForced)
-            m_runMode = isConsoleApplication() ? ApplicationLauncher::Console
-                                               : ApplicationLauncher::Gui;
+        auto terminalAspect = extraAspect<TerminalAspect>();
+        if (success && !terminalAspect->isUserSet())
+            terminalAspect->setUseTerminal(isConsoleApplication());
         emit enabledChanged();
     });
+    connect(BuildManager::instance(), &BuildManager::buildStateChanged, this,
+            [this, project](Project *p) {
+                if (p == project && !BuildManager::isBuilding(p))
+                    emit enabledChanged();
+            }
+    );
 
     connect(target(), &Target::activeDeployConfigurationChanged,
             this, &QbsRunConfiguration::installStepChanged);
@@ -178,30 +182,6 @@ QWidget *QbsRunConfiguration::createConfigurationWidget()
     return new QbsRunConfigurationWidget(this, 0);
 }
 
-QVariantMap QbsRunConfiguration::toMap() const
-{
-    QVariantMap map(LocalApplicationRunConfiguration::toMap());
-    map.insert(QLatin1String(COMMAND_LINE_ARGUMENTS_KEY), m_commandLineArguments);
-    if (m_runModeForced)
-        map.insert(QLatin1String(USE_TERMINAL_KEY), m_runMode == ApplicationLauncher::Console);
-    map.insert(QLatin1String(USER_WORKING_DIRECTORY_KEY), m_userWorkingDirectory);
-    return map;
-}
-
-bool QbsRunConfiguration::fromMap(const QVariantMap &map)
-{
-    m_commandLineArguments = map.value(QLatin1String(COMMAND_LINE_ARGUMENTS_KEY)).toString();
-    if (map.contains(QLatin1String(USE_TERMINAL_KEY))) {
-        m_runMode = map.value(QLatin1String(USE_TERMINAL_KEY), false).toBool() ?
-                    ApplicationLauncher::Console : ApplicationLauncher::Gui;
-        m_runModeForced = true;
-    }
-
-    m_userWorkingDirectory = map.value(QLatin1String(USER_WORKING_DIRECTORY_KEY)).toString();
-
-    return RunConfiguration::fromMap(map);
-}
-
 void QbsRunConfiguration::installStepChanged()
 {
     if (m_currentInstallStep)
@@ -254,7 +234,7 @@ QString QbsRunConfiguration::executable() const
 
 ApplicationLauncher::Mode QbsRunConfiguration::runMode() const
 {
-    return m_runMode;
+    return extraAspect<TerminalAspect>()->runMode();
 }
 
 bool QbsRunConfiguration::isConsoleApplication() const
@@ -266,19 +246,13 @@ bool QbsRunConfiguration::isConsoleApplication() const
 
 QString QbsRunConfiguration::workingDirectory() const
 {
-    EnvironmentAspect *aspect = extraAspect<EnvironmentAspect>();
-    QTC_ASSERT(aspect, return baseWorkingDirectory());
-    return QDir::cleanPath(aspect->environment().expandVariables(
-                macroExpander()->expand(baseWorkingDirectory())));
+    const auto *wdAspect = extraAspect<WorkingDirectoryAspect>();
+    QTC_ASSERT(wdAspect, return baseWorkingDirectory());
+    return wdAspect->workingDirectory();
 }
 
 QString QbsRunConfiguration::baseWorkingDirectory() const
 {
-    // if the user overrode us, then return his working directory
-    if (!m_userWorkingDirectory.isEmpty())
-        return m_userWorkingDirectory;
-
-    // else what the pro file reader tells us
     const QString exe = executable();
     if (!exe.isEmpty())
         return QFileInfo(executable()).absolutePath();
@@ -287,39 +261,12 @@ QString QbsRunConfiguration::baseWorkingDirectory() const
 
 QString QbsRunConfiguration::commandLineArguments() const
 {
-    return macroExpander()->expandProcessArgs(m_commandLineArguments);
-}
-
-QString QbsRunConfiguration::rawCommandLineArguments() const
-{
-    return m_commandLineArguments;
-}
-
-void QbsRunConfiguration::setBaseWorkingDirectory(const QString &wd)
-{
-    const QString &oldWorkingDirectory = workingDirectory();
-
-    m_userWorkingDirectory = wd;
-
-    const QString &newWorkingDirectory = workingDirectory();
-    if (oldWorkingDirectory != newWorkingDirectory)
-        emit baseWorkingDirectoryChanged(newWorkingDirectory);
-}
-
-void QbsRunConfiguration::setCommandLineArguments(const QString &argumentsString)
-{
-    m_commandLineArguments = argumentsString;
-    emit commandLineArgumentsChanged(argumentsString);
+    return extraAspect<ArgumentsAspect>()->arguments();
 }
 
 void QbsRunConfiguration::setRunMode(ApplicationLauncher::Mode runMode)
 {
-    if (m_runMode == runMode)
-        return;
-
-    m_runModeForced = true;
-    m_runMode = runMode;
-    emit runModeChanged(runMode);
+    extraAspect<TerminalAspect>()->setRunMode(runMode);
 }
 
 void QbsRunConfiguration::addToBaseEnvironment(Utils::Environment &env) const
@@ -329,7 +276,6 @@ void QbsRunConfiguration::addToBaseEnvironment(Utils::Environment &env) const
         const qbs::ProductData product = findProduct(project->qbsProjectData(), m_uniqueProductName);
         if (product.isValid()) {
             QProcessEnvironment procEnv = env.toProcessEnvironment();
-            procEnv.insert(QLatin1String("QBS_INSTALL_ROOT"), installRoot());
             procEnv.insert(QLatin1String("QBS_RUN_FILE_PATH"), executable());
             qbs::RunEnvironment qbsRunEnv = project->qbsProject().getRunEnvironment(product, installOptions(),
                     procEnv, QbsManager::settings());
@@ -412,63 +358,18 @@ QbsRunConfigurationWidget::QbsRunConfigurationWidget(QbsRunConfiguration *rc, QW
     toplayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
     toplayout->setMargin(0);
 
-    m_executableLineEdit = new QLineEdit(this);
-    m_executableLineEdit->setEnabled(false);
-    m_executableLineEdit->setPlaceholderText(tr("<unknown>"));
-    toplayout->addRow(tr("Executable:"), m_executableLineEdit);
+    m_executableLineLabel = new QLabel(this);
+    m_executableLineLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);
+    setExecutableLineText();
+    toplayout->addRow(tr("Executable:"), m_executableLineLabel);
 
-    QLabel *argumentsLabel = new QLabel(tr("Arguments:"), this);
-    m_argumentsLineEdit = new QLineEdit(m_rc->rawCommandLineArguments(), this);
-    argumentsLabel->setBuddy(m_argumentsLineEdit);
-    toplayout->addRow(argumentsLabel, m_argumentsLineEdit);
+    m_rc->extraAspect<ArgumentsAspect>()->addToMainConfigurationWidget(this, toplayout);
+    m_rc->extraAspect<WorkingDirectoryAspect>()->addToMainConfigurationWidget(this, toplayout);
 
-    m_workingDirectoryEdit = new Utils::PathChooser(this);
-    m_workingDirectoryEdit->setHistoryCompleter(QLatin1String("Qbs.WorkingDir.History"));
-    m_workingDirectoryEdit->setExpectedKind(Utils::PathChooser::Directory);
-    EnvironmentAspect *aspect = m_rc->extraAspect<EnvironmentAspect>();
-    if (aspect) {
-        connect(aspect, SIGNAL(environmentChanged()), this, SLOT(environmentWasChanged()));
-        environmentWasChanged();
-    }
-    m_workingDirectoryEdit->setPromptDialogTitle(tr("Select Working Directory"));
-
-    QToolButton *resetButton = new QToolButton(this);
-    resetButton->setToolTip(tr("Reset to default"));
-    resetButton->setIcon(QIcon(QLatin1String(Core::Constants::ICON_RESET)));
-
-    QHBoxLayout *boxlayout = new QHBoxLayout();
-    boxlayout->setMargin(0);
-    boxlayout->addWidget(m_workingDirectoryEdit);
-    boxlayout->addWidget(resetButton);
-    toplayout->addRow(tr("Working directory:"), boxlayout);
-
-    QHBoxLayout *innerBox = new QHBoxLayout();
-    m_useTerminalCheck = new QCheckBox(tr("Run in terminal"), this);
-    innerBox->addWidget(m_useTerminalCheck);
-
-    innerBox->addStretch();
-    toplayout->addRow(QString(), innerBox);
+    m_rc->extraAspect<TerminalAspect>()->addToMainConfigurationWidget(this, toplayout);
 
     runConfigurationEnabledChange();
 
-    connect(m_workingDirectoryEdit, SIGNAL(changed(QString)),
-            this, SLOT(workDirectoryEdited()));
-
-    connect(resetButton, SIGNAL(clicked()),
-            this, SLOT(workingDirectoryWasReset()));
-
-    connect(m_argumentsLineEdit, SIGNAL(textEdited(QString)),
-            this, SLOT(argumentsEdited(QString)));
-    connect(m_useTerminalCheck, SIGNAL(toggled(bool)),
-            this, SLOT(termToggled(bool)));
-
-    connect(m_rc, SIGNAL(baseWorkingDirectoryChanged(QString)),
-            this, SLOT(workingDirectoryChanged(QString)));
-
-    connect(m_rc, SIGNAL(commandLineArgumentsChanged(QString)),
-            this, SLOT(commandLineArgumentsChanged(QString)));
-    connect(m_rc, SIGNAL(runModeChanged(ProjectExplorer::ApplicationLauncher::Mode)),
-            this, SLOT(runModeChanged(ProjectExplorer::ApplicationLauncher::Mode)));
     connect(m_rc, SIGNAL(targetInformationChanged()),
             this, SLOT(targetInformationHasChanged()), Qt::QueuedConnection);
 
@@ -476,13 +377,6 @@ QbsRunConfigurationWidget::QbsRunConfigurationWidget(QbsRunConfiguration *rc, QW
             this, SLOT(runConfigurationEnabledChange()));
 }
 
-void QbsRunConfigurationWidget::environmentWasChanged()
-{
-    EnvironmentAspect *aspect = m_rc->extraAspect<EnvironmentAspect>();
-    QTC_ASSERT(aspect, return);
-    m_workingDirectoryEdit->setEnvironment(aspect->environment());
-}
-
 void QbsRunConfigurationWidget::runConfigurationEnabledChange()
 {
     bool enabled = m_rc->isEnabled();
@@ -490,67 +384,24 @@ void QbsRunConfigurationWidget::runConfigurationEnabledChange()
     m_disabledReason->setVisible(!enabled);
     m_disabledReason->setText(m_rc->disabledReason());
 
-    m_useTerminalCheck->setChecked(m_rc->runMode() == ApplicationLauncher::Console);
     targetInformationHasChanged();
 }
 
-void QbsRunConfigurationWidget::workDirectoryEdited()
-{
-    if (m_ignoreChange)
-        return;
-    m_ignoreChange = true;
-    m_rc->setBaseWorkingDirectory(m_workingDirectoryEdit->rawPath());
-    m_ignoreChange = false;
-}
-
-void QbsRunConfigurationWidget::workingDirectoryWasReset()
-{
-    // This emits a signal connected to workingDirectoryChanged()
-    // that sets the m_workingDirectoryEdit
-    m_rc->setBaseWorkingDirectory(QString());
-}
-
-void QbsRunConfigurationWidget::argumentsEdited(const QString &args)
-{
-    m_ignoreChange = true;
-    m_rc->setCommandLineArguments(args);
-    m_ignoreChange = false;
-}
-
-void QbsRunConfigurationWidget::termToggled(bool on)
-{
-    m_ignoreChange = true;
-    m_rc->setRunMode(on ? ApplicationLauncher::Console : ApplicationLauncher::Gui);
-    m_ignoreChange = false;
-}
-
 void QbsRunConfigurationWidget::targetInformationHasChanged()
 {
     m_ignoreChange = true;
-    m_executableLineEdit->setText(m_rc->executable());
+    setExecutableLineText(m_rc->executable());
 
-    m_workingDirectoryEdit->setPath(m_rc->baseWorkingDirectory());
-    m_workingDirectoryEdit->setBaseFileName(m_rc->target()->project()->projectDirectory());
+    WorkingDirectoryAspect *aspect = m_rc->extraAspect<WorkingDirectoryAspect>();
+    aspect->setDefaultWorkingDirectory(m_rc->baseWorkingDirectory());
+    aspect->pathChooser()->setBaseFileName(m_rc->target()->project()->projectDirectory());
     m_ignoreChange = false;
 }
 
-void QbsRunConfigurationWidget::workingDirectoryChanged(const QString &workingDirectory)
+void QbsRunConfigurationWidget::setExecutableLineText(const QString &text)
 {
-    if (!m_ignoreChange)
-        m_workingDirectoryEdit->setPath(workingDirectory);
-}
-
-void QbsRunConfigurationWidget::commandLineArgumentsChanged(const QString &args)
-{
-    if (m_ignoreChange)
-        return;
-    m_argumentsLineEdit->setText(args);
-}
-
-void QbsRunConfigurationWidget::runModeChanged(ApplicationLauncher::Mode runMode)
-{
-    if (!m_ignoreChange)
-        m_useTerminalCheck->setChecked(runMode == ApplicationLauncher::Console);
+    const QString newText = text.isEmpty() ? tr("<unknown>") : text;
+    m_executableLineLabel->setText(newText);
 }
 
 // --------------------------------------------------------------------
