@@ -75,22 +75,13 @@ const char QMAKE_QMLDEBUGLIB_KEY[] = "QtProjectManager.QMakeBuildStep.LinkQmlDeb
 }
 
 QMakeStep::QMakeStep(BuildStepList *bsl) :
-    AbstractProcessStep(bsl, Core::Id(QMAKE_BS_ID)),
-    m_forced(false),
-    m_needToRunQMake(false),
-    m_linkQmlDebuggingLibrary(DebugLink),
-    m_useQtQuickCompiler(false),
-    m_separateDebugInfo(false)
+    AbstractProcessStep(bsl, Core::Id(QMAKE_BS_ID))
 {
     ctor();
 }
 
 QMakeStep::QMakeStep(BuildStepList *bsl, Core::Id id) :
-    AbstractProcessStep(bsl, id),
-    m_forced(false),
-    m_linkQmlDebuggingLibrary(DebugLink),
-    m_useQtQuickCompiler(false),
-    m_separateDebugInfo(false)
+    AbstractProcessStep(bsl, id)
 {
     ctor();
 }
@@ -178,11 +169,8 @@ QMakeStepConfig QMakeStep::deducedArguments()
 
     config.archConfig = QMakeStepConfig::targetArchFor(targetAbi, version);
     config.osType = QMakeStepConfig::osTypeFor(targetAbi, version);
-    if (linkQmlDebuggingLibrary() && version) {
-        config.linkQmlDebuggingQQ1 = true;
-        if (version->qtVersion().majorVersion >= 5)
-            config.linkQmlDebuggingQQ2 = true;
-    }
+    if (linkQmlDebuggingLibrary() && version && version->qtVersion().majorVersion >= 5)
+        config.linkQmlDebuggingQQ2 = true;
 
     if (useQtQuickCompiler() && version)
         config.useQtQuickCompiler = true;
@@ -336,23 +324,15 @@ void QMakeStep::setUserArguments(const QString &arguments)
 
 bool QMakeStep::linkQmlDebuggingLibrary() const
 {
-    if (m_linkQmlDebuggingLibrary == DoLink)
-        return true;
-    if (m_linkQmlDebuggingLibrary == DoNotLink)
-        return false;
-
-    const Core::Context languages = project()->projectLanguages();
-    if (!languages.contains(ProjectExplorer::Constants::LANG_QMLJS))
-        return false;
-    return (qmakeBuildConfiguration()->buildType() & BuildConfiguration::Debug);
+    return m_linkQmlDebuggingLibrary;
 }
 
 void QMakeStep::setLinkQmlDebuggingLibrary(bool enable)
 {
-    if ((enable && (m_linkQmlDebuggingLibrary == DoLink))
-            || (!enable && (m_linkQmlDebuggingLibrary == DoNotLink)))
+    if (enable == m_linkQmlDebuggingLibrary)
         return;
-    m_linkQmlDebuggingLibrary = enable ? DoLink : DoNotLink;
+
+    m_linkQmlDebuggingLibrary = enable;
 
     emit linkQmlDebuggingLibraryChanged();
 
@@ -426,8 +406,7 @@ QVariantMap QMakeStep::toMap() const
 {
     QVariantMap map(AbstractProcessStep::toMap());
     map.insert(QLatin1String(QMAKE_ARGUMENTS_KEY), m_userArgs);
-    map.insert(QLatin1String(QMAKE_QMLDEBUGLIBAUTO_KEY), m_linkQmlDebuggingLibrary == DebugLink);
-    map.insert(QLatin1String(QMAKE_QMLDEBUGLIB_KEY), m_linkQmlDebuggingLibrary == DoLink);
+    map.insert(QLatin1String(QMAKE_QMLDEBUGLIB_KEY), m_linkQmlDebuggingLibrary);
     map.insert(QLatin1String(QMAKE_FORCED_KEY), m_forced);
     map.insert(QLatin1String(QMAKE_USE_QTQUICKCOMPILER), m_useQtQuickCompiler);
     map.insert(QLatin1String(QMAKE_SEPARATEDEBUGINFO_KEY), m_separateDebugInfo);
@@ -439,13 +418,15 @@ bool QMakeStep::fromMap(const QVariantMap &map)
     m_userArgs = map.value(QLatin1String(QMAKE_ARGUMENTS_KEY)).toString();
     m_forced = map.value(QLatin1String(QMAKE_FORCED_KEY), false).toBool();
     m_useQtQuickCompiler = map.value(QLatin1String(QMAKE_USE_QTQUICKCOMPILER), false).toBool();
+
+    // QMAKE_QMLDEBUGLIBAUTO_KEY was used in versions 2.3 to 3.5 (both included) to automatically
+    // change the qml_debug CONFIG flag based no the qmake build configuration.
     if (map.value(QLatin1String(QMAKE_QMLDEBUGLIBAUTO_KEY), false).toBool()) {
-        m_linkQmlDebuggingLibrary = DebugLink;
+        m_linkQmlDebuggingLibrary =
+                project()->projectLanguages().contains(ProjectExplorer::Constants::LANG_QMLJS) &&
+                (qmakeBuildConfiguration()->qmakeBuildConfiguration() & BaseQtVersion::DebugBuild);
     } else {
-        if (map.value(QLatin1String(QMAKE_QMLDEBUGLIB_KEY), false).toBool())
-            m_linkQmlDebuggingLibrary = DoLink;
-        else
-            m_linkQmlDebuggingLibrary = DoNotLink;
+        m_linkQmlDebuggingLibrary = map.value(QLatin1String(QMAKE_QMLDEBUGLIB_KEY), false).toBool();
     }
     m_separateDebugInfo = map.value(QLatin1String(QMAKE_SEPARATEDEBUGINFO_KEY), false).toBool();
 
@@ -457,8 +438,7 @@ bool QMakeStep::fromMap(const QVariantMap &map)
 ////
 
 QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
-    : BuildStepConfigWidget(), m_ui(new Internal::Ui::QMakeStep), m_step(step),
-      m_ignoreChange(false)
+    : BuildStepConfigWidget(), m_ui(new Internal::Ui::QMakeStep), m_step(step)
 {
     m_ui->setupUi(this);
 
@@ -878,8 +858,6 @@ QStringList QMakeStepConfig::toArguments() const
     else if (osType == IphoneOS)
         arguments << QLatin1String("CONFIG+=iphoneos");
 
-    if (linkQmlDebuggingQQ1)
-        arguments << QLatin1String("CONFIG+=declarative_debug");
     if (linkQmlDebuggingQQ2)
         arguments << QLatin1String("CONFIG+=qml_debug");
 
