@@ -129,12 +129,12 @@ QtProjectParameters::Type LibraryIntroPage::type() const
 }
 
 // ------------------- LibraryWizardDialog
-LibraryWizardDialog::LibraryWizardDialog(const QString &templateName,
+LibraryWizardDialog::LibraryWizardDialog(const Core::BaseFileWizardFactory *factory,
+                                         const QString &templateName,
                                          const QIcon &icon,
-                                         bool showModulesPage,
                                          QWidget *parent,
                                          const Core::WizardDialogParameters &parameters) :
-    BaseQmakeProjectWizardDialog(showModulesPage, new LibraryIntroPage, -1, parent, parameters),
+    BaseQmakeProjectWizardDialog(factory, true, new LibraryIntroPage, -1, parent, parameters),
     m_filesPage(new FilesPage),
     m_pluginBaseClassesInitialized(false),
     m_filesPageId(-1), m_modulesPageId(-1), m_targetPageId(-1)
@@ -178,9 +178,9 @@ LibraryWizardDialog::LibraryWizardDialog(const QString &templateName,
         }
     }
 
-    connect(this, SIGNAL(currentIdChanged(int)), this, SLOT(slotCurrentIdChanged(int)));
+    connect(this, &QWizard::currentIdChanged, this, &LibraryWizardDialog::slotCurrentIdChanged);
 
-    addExtensionPages(parameters.extensionPages());
+    addExtensionPages(extensionPages());
 }
 
 void LibraryWizardDialog::setSuffixes(const QString &header, const QString &source,  const QString &form)
@@ -297,14 +297,15 @@ void LibraryWizardDialog::setupFilesPage()
         }
         m_filesPage->setBaseClassInputVisible(true);
         break;
-    default: {
-        // Urrm, figure out a good class name. Use project name this time
-        QString className = projectName();
-        if (!className.isEmpty())
-            className[0] = className.at(0).toUpper();
-        m_filesPage->setClassName(className);
-        m_filesPage->setBaseClassInputVisible(false);
-    }
+    default:
+        if (!m_filesPage->isComplete()) {
+            // Urrm, figure out a good class name. Use project name this time
+            QString className = projectName();
+            if (!className.isEmpty())
+                className[0] = className.at(0).toUpper();
+            m_filesPage->setClassName(className);
+            m_filesPage->setBaseClassInputVisible(false);
+        }
         break;
     }
 }
