@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ****************************************************************************/
 
@@ -41,6 +37,7 @@
 #include <qmljs/qmljsscopechain.h>
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
+#include <qmljs/qmljsvalueowner.h>
 #include <languageutils/fakemetaobject.h>
 
 namespace QmlDesigner {
@@ -134,10 +131,10 @@ static QString qualifiedTypeNameForContext(const ObjectValue *objectValue,
         foreach (const LanguageUtils::FakeMetaObject::Export &e, cppComponent->metaObject()->exports()) {
             if (e.type == className)
                 packages << e.package;
-            if (e.package == QmlJS::CppQmlTypes::cppPackage)
+            if (e.package == CppQmlTypes::cppPackage)
                 cppName = e.type;
         }
-        if (packages.size() == 1 && packages.at(0) == QmlJS::CppQmlTypes::cppPackage)
+        if (packages.size() == 1 && packages.at(0) == CppQmlTypes::cppPackage)
             return packages.at(0) + QLatin1Char('.') + className;
     }
     // try to recover a "global context name"
@@ -157,7 +154,7 @@ static QString qualifiedTypeNameForContext(const ObjectValue *objectValue,
                 case ImportType::Library:
                 {
                     QString typeName = objectValue->className();
-                    if (!e.typeName.isEmpty() && e.typeName != Export::LibraryTypeName) {
+                    if (!e.typeName.isEmpty() && e.typeName != Export::libraryTypeName()) {
                         typeName = e.typeName;
                         if (typeName != objectValue->className())
                             qCWarning(qmljsLog) << "Outdated classname " << objectValue->className()
@@ -263,7 +260,7 @@ static QString qualifiedTypeNameForContext(const ObjectValue *objectValue,
             return optimalName(possibleFiles);
     } while (false);
     if (!cppName.isEmpty())
-        return QmlJS::CppQmlTypes::cppPackage + QLatin1Char('.') + cppName;
+        return CppQmlTypes::cppPackage + QLatin1Char('.') + cppName;
     if (const CppComponentValue *cppComponent = value_cast<CppComponentValue>(objectValue)) {
         if (cppComponent->moduleName().isEmpty())
             return cppComponent->className();
@@ -279,7 +276,7 @@ class PropertyMemberProcessor : public MemberProcessor
 public:
     PropertyMemberProcessor(const ContextPtr &context) : m_context(context)
     {}
-    bool processProperty(const QString &name, const Value *value)
+    bool processProperty(const QString &name, const Value *value, const QmlJS::PropertyInfo &) override
     {
         PropertyName propertyName = name.toUtf8();
         const ASTPropertyReference *ref = value_cast<ASTPropertyReference>(value);
@@ -317,7 +314,7 @@ public:
         return true;
     }
 
-    virtual bool processSignal(const QString &name, const Value * /*value*/)
+    bool processSignal(const QString &name, const Value * /*value*/) override
     {
         m_signals.append(name.toUtf8());
         return true;
@@ -566,13 +563,13 @@ public:
 private:
     NodeMetaInfoPrivate(Model *model, TypeName type, int maj = -1, int min = -1);
 
-    const QmlJS::CppComponentValue *getCppComponentValue() const;
-    const QmlJS::ObjectValue *getObjectValue() const;
+    const CppComponentValue *getCppComponentValue() const;
+    const ObjectValue *getObjectValue() const;
     void setupPropertyInfo(QList<PropertyInfo> propertyInfos);
     void setupLocalPropertyInfo(QList<PropertyInfo> propertyInfos);
     QString lookupName() const;
     QStringList lookupNameComponent() const;
-    const QmlJS::CppComponentValue *getNearestCppComponentValue() const;
+    const CppComponentValue *getNearestCppComponentValue() const;
     QString fullQualifiedImportAliasType() const;
 
     TypeName m_qualfiedTypeName;
@@ -590,7 +587,7 @@ private:
     QSet<QString> m_prototypeCacheNegatives;
 
     //storing the pointer would not be save
-    QmlJS::ContextPtr context() const;
+    ContextPtr context() const;
     const Document *document() const;
 
     QPointer<Model> m_model;
@@ -720,7 +717,7 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, TypeName type, int maj, i
     }
 }
 
-const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() const
+const CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() const
 {
     const QList<TypeName> nameComponents = m_qualfiedTypeName.split('.');
     if (nameComponents.size() < 2)
@@ -747,29 +744,44 @@ const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() cons
             return cppValue;
     }
 
-    const QmlJS::CppComponentValue *value = value_cast<CppComponentValue>(getObjectValue());
+    const CppComponentValue *value = value_cast<CppComponentValue>(getObjectValue());
     if (value)
         return value;
 
     // maybe 'type' is a cpp name
-    const QmlJS::CppComponentValue *cppValue = context()->valueOwner()->cppQmlTypes().objectByCppName(type);
+    const CppComponentValue *cppValue = context()->valueOwner()->cppQmlTypes().objectByCppName(type);
+
+    if (cppValue) {
+        foreach (const LanguageUtils::FakeMetaObject::Export &exportValue, cppValue->metaObject()->exports()) {
+            if (exportValue.package.toUtf8() != "<cpp>") {
+                foreach (const QmlJS::Import &import, context()->imports(document())->all()) {
+                    if (import.info.path() != exportValue.package)
+                        continue;
+                    const Value *lookupResult = import.object->lookupMember(exportValue.type, context());
+                    const CppComponentValue *cppValue = value_cast<CppComponentValue>(lookupResult);
+                    if (cppValue)
+                        return cppValue;
+                }
+            }
+        }
+    }
 
     return cppValue;
 }
 
-const QmlJS::ObjectValue *NodeMetaInfoPrivate::getObjectValue() const
+const ObjectValue *NodeMetaInfoPrivate::getObjectValue() const
 {
     return context()->lookupType(document(), lookupNameComponent());
 }
 
-QmlJS::ContextPtr NodeMetaInfoPrivate::context() const
+ContextPtr NodeMetaInfoPrivate::context() const
 {
     if (m_model && m_model->rewriterView() && m_model->rewriterView()->scopeChain())
         return m_model->rewriterView()->scopeChain()->context();
-    return QmlJS::ContextPtr(0);
+    return ContextPtr(0);
 }
 
-const QmlJS::Document *NodeMetaInfoPrivate::document() const
+const Document *NodeMetaInfoPrivate::document() const
 {
     if (m_model && m_model->rewriterView())
         return m_model->rewriterView()->document();
@@ -786,8 +798,10 @@ void NodeMetaInfoPrivate::setupLocalPropertyInfo(QList<PropertyInfo> localProper
 void NodeMetaInfoPrivate::setupPropertyInfo(QList<PropertyInfo> propertyInfos)
 {
     foreach (const PropertyInfo &propertyInfo, propertyInfos) {
-        m_properties.append(propertyInfo.first);
-        m_propertyTypes.append(propertyInfo.second);
+        if (!m_properties.contains(propertyInfo.first)) {
+            m_properties.append(propertyInfo.first);
+            m_propertyTypes.append(propertyInfo.second);
+        }
     }
 }
 
@@ -812,7 +826,7 @@ bool NodeMetaInfoPrivate::isPropertyWritable(const PropertyName &propertyName) c
             return true;
     }
 
-    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return true;
     if (qmlObjectValue->hasProperty(propertyName))
@@ -843,7 +857,7 @@ bool NodeMetaInfoPrivate::isPropertyList(const PropertyName &propertyName) const
             return true;
     }
 
-    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
     return qmlObjectValue->isListProperty(propertyName);
@@ -870,7 +884,7 @@ bool NodeMetaInfoPrivate::isPropertyPointer(const PropertyName &propertyName) co
             return true;
     }
 
-    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
     return qmlObjectValue->isPointer(propertyName);
@@ -968,7 +982,7 @@ static QString getPackage(const QString &name)
         return QString();
     nameComponents.removeLast();
 
-    return nameComponents.join(QStringLiteral("."));
+    return nameComponents.join(QLatin1Char('.'));
 }
 
 bool NodeMetaInfoPrivate::cleverCheckType(const QString &otherType) const
@@ -1117,6 +1131,11 @@ QString NodeMetaInfoPrivate::importDirectoryPath() const
                     const QString targetPath = QDir(importPath).filePath(importInfo.path());
                     if (QDir(targetPath).exists())
                         return targetPath;
+                    const QString targetPathVersion = QDir(importPath).filePath(importInfo.path()
+                                                                                + QLatin1Char('.')
+                                                                                + QString::number(importInfo.version().majorVersion()));
+                    if (QDir(targetPathVersion).exists())
+                        return targetPathVersion;
                 }
             }
         }
@@ -1132,7 +1151,7 @@ QString NodeMetaInfoPrivate::lookupName() const
     QStringList packageClassName = className.split(QLatin1Char('.'));
     if (packageClassName.size() > 1) {
         className = packageClassName.takeLast();
-        packageName = packageClassName.join(QStringLiteral("."));
+        packageName = packageClassName.join(QLatin1Char('.'));
     }
 
     return CppQmlTypes::qualifiedName(
@@ -1221,7 +1240,7 @@ void NodeMetaInfoPrivate::setupPrototypes()
                     QString uri = importInfo.name();
                     uri.replace(QStringLiteral(","), QStringLiteral("."));
                     if (!uri.isEmpty())
-                        description.className = QString(uri + QString::fromLatin1(".") + QString::fromLatin1(description.className)).toLatin1();
+                        description.className = QString(uri + QString::fromLatin1(".") + QString::fromUtf8(description.className)).toUtf8();
                 }
 
                 m_prototypes.append(description);
@@ -1236,7 +1255,7 @@ QList<TypeDescription> NodeMetaInfoPrivate::prototypes() const
     return m_prototypes;
 }
 
-const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getNearestCppComponentValue() const
+const CppComponentValue *NodeMetaInfoPrivate::getNearestCppComponentValue() const
 {
     if (m_isFileComponent)
         return findQmlPrototype(getObjectValue(), context());
