@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -125,20 +125,21 @@ void QmlProfilerEventsModelProxy::clear()
 
 void QmlProfilerEventsModelProxy::limitToRange(qint64 rangeStart, qint64 rangeEnd)
 {
-    loadData(rangeStart, rangeEnd);
+    if (!d->modelManager->isEmpty())
+        loadData(rangeStart, rangeEnd);
 }
 
 void QmlProfilerEventsModelProxy::dataChanged()
 {
-    if (d->modelManager->state() == QmlProfilerDataState::ProcessingData)
+    if (d->modelManager->state() == QmlProfilerModelManager::ProcessingData)
         loadData();
-    else if (d->modelManager->state() == QmlProfilerDataState::ClearingData)
+    else if (d->modelManager->state() == QmlProfilerModelManager::ClearingData)
         clear();
 }
 
 void QmlProfilerEventsModelProxy::notesChanged(int typeIndex)
 {
-    const NotesModel *notesModel = d->modelManager->notesModel();
+    const QmlProfilerNotesModel *notesModel = d->modelManager->notesModel();
     if (typeIndex == -1) {
         d->notes.clear();
         for (int noteId = 0; noteId < notesModel->count(); ++noteId) {
@@ -154,7 +155,7 @@ void QmlProfilerEventsModelProxy::notesChanged(int typeIndex)
         }
     } else {
         d->notes.remove(typeIndex);
-        QVariantList changedNotes = notesModel->byTypeId(typeIndex);
+        const QVariantList changedNotes = notesModel->byTypeId(typeIndex);
         if (!changedNotes.isEmpty()) {
             QStringList newNotes;
             for (QVariantList::ConstIterator it = changedNotes.begin(); it !=  changedNotes.end();
@@ -289,7 +290,6 @@ int QmlProfilerEventsModelProxy::count() const
     return d->data.count();
 }
 
-//////////////////////////////////////////////////////////////////////////////////
 QmlProfilerEventRelativesModelProxy::QmlProfilerEventRelativesModelProxy(QmlProfilerModelManager *modelManager,
                                                                          QmlProfilerEventsModelProxy *eventsModel,
                                                                          QObject *parent)
@@ -343,8 +343,6 @@ void QmlProfilerEventRelativesModelProxy::dataChanged()
     emit dataAvailable();
 }
 
-
-//////////////////////////////////////////////////////////////////////////////////
 QmlProfilerEventParentsModelProxy::QmlProfilerEventParentsModelProxy(QmlProfilerModelManager *modelManager,
                                                                      QmlProfilerEventsModelProxy *eventsModel,
                                                                      QObject *parent)
@@ -409,7 +407,6 @@ void QmlProfilerEventParentsModelProxy::loadData()
     }
 }
 
-//////////////////////////////////////////////////////////////////////////////////
 QmlProfilerEventChildrenModelProxy::QmlProfilerEventChildrenModelProxy(QmlProfilerModelManager *modelManager,
                                                                        QmlProfilerEventsModelProxy *eventsModel,
                                                                        QObject *parent)
