@@ -33,18 +33,20 @@
 #include "qmlprojectmanagerconstants.h"
 #include "qmlprojectrunconfigurationwidget.h"
 #include "qmlprojectenvironmentaspect.h"
-#include <coreplugin/mimedatabase.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/idocument.h>
 #include <projectexplorer/target.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtoutputformatter.h>
 #include <qtsupport/qtsupportconstants.h>
 
 #include <utils/fileutils.h>
+#include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcprocess.h>
 #include <utils/winutils.h>
+#include <qmljstools/qmljstoolsconstants.h>
 
 using namespace Core;
 using namespace ProjectExplorer;
@@ -103,6 +105,7 @@ void QmlProjectRunConfiguration::ctor()
         setDisplayName(tr("QML Scene", "QMLRunConfiguration display name."));
     else
         setDisplayName(tr("QML Viewer", "QMLRunConfiguration display name."));
+    updateEnabled();
 }
 
 QString QmlProjectRunConfiguration::executable() const
@@ -257,7 +260,7 @@ void QmlProjectRunConfiguration::changeCurrentFile(IEditor *editor)
         editor = EditorManager::currentEditor();
 
     if (editor)
-        m_currentFileFilename = editor->document()->filePath();
+        m_currentFileFilename = editor->document()->filePath().toString();
     updateEnabled();
 }
 
@@ -265,21 +268,23 @@ void QmlProjectRunConfiguration::updateEnabled()
 {
     bool qmlFileFound = false;
     if (mainScriptSource() == FileInEditor) {
+        Utils::MimeDatabase mimeDataBase;
         IDocument *document = EditorManager::currentDocument();
+        Utils::MimeType mainScriptMimeType = mimeDataBase.mimeTypeForFile(mainScript());
         if (document) {
-            m_currentFileFilename = document->filePath();
-            if (MimeDatabase::findByFile(mainScript()).type() == QLatin1String("application/x-qml"))
+            m_currentFileFilename = document->filePath().toString();
+            if (mainScriptMimeType.matchesName(QLatin1String(ProjectExplorer::Constants::QML_MIMETYPE)))
                 qmlFileFound = true;
         }
         if (!document
-                || MimeDatabase::findByFile(mainScript()).type() == QLatin1String("application/x-qmlproject")) {
+                || mainScriptMimeType.matchesName(QLatin1String(QmlJSTools::Constants::QMLPROJECT_MIMETYPE))) {
             // find a qml file with lowercase filename. This is slow, but only done
             // in initialization/other border cases.
             foreach (const QString &filename, target()->project()->files(Project::AllFiles)) {
                 const QFileInfo fi(filename);
 
                 if (!filename.isEmpty() && fi.baseName()[0].isLower()
-                        && MimeDatabase::findByFile(fi).type() == QLatin1String("application/x-qml"))
+                        && mimeDataBase.mimeTypeForFile(fi).matchesName(QLatin1String(ProjectExplorer::Constants::QML_MIMETYPE)))
                 {
                     m_currentFileFilename = filename;
                     qmlFileFound = true;
@@ -293,17 +298,16 @@ void QmlProjectRunConfiguration::updateEnabled()
     }
 
     bool newValue = QFileInfo::exists(executable()) && qmlFileFound;
+    m_isEnabled = newValue;
 
     // Always emit change signal to force reevaluation of run/debug buttons
-    m_isEnabled = newValue;
     emit enabledChanged();
 }
 
 bool QmlProjectRunConfiguration::isValidVersion(QtSupport::BaseQtVersion *version)
 {
     if (version
-            && (version->type() == QLatin1String(QtSupport::Constants::DESKTOPQT)
-                || version->type() == QLatin1String(QtSupport::Constants::SIMULATORQT))
+            && version->type() == QLatin1String(QtSupport::Constants::DESKTOPQT)
             && !version->qmlviewerCommand().isEmpty()) {
         return true;
     }
