@@ -38,20 +38,20 @@
 #include "qtkitinformation.h"
 #include "qtoptionspage.h"
 #include "qtversionmanager.h"
-#include "simulatorqtversionfactory.h"
 #include "uicodemodelsupport.h"
 #include "winceqtversionfactory.h"
 
 #include "profilereader.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/mimedatabase.h>
 #include <coreplugin/jsexpander.h>
 
 #include <projectexplorer/project.h>
-#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/target.h>
+
 #include <utils/macroexpander.h>
+#include <utils/mimetypes/mimedatabase.h>
 
 #include <QtPlugin>
 
@@ -70,14 +70,12 @@ bool QtSupportPlugin::initialize(const QStringList &arguments, QString *errorMes
     ProFileEvaluator::initialize();
     new ProFileCacheManager(this);
 
-    if (!MimeDatabase::addMimeTypes(QLatin1String(":qtsupport/QtSupport.mimetypes.xml"), errorMessage))
-        return false;
+    Utils::MimeDatabase::addMimeTypes(QLatin1String(":qtsupport/QtSupport.mimetypes.xml"));
 
     JsExpander::registerQObjectForJs(QLatin1String("QtSupport"), new CodeGenerator);
 
     addAutoReleasedObject(new QtVersionManager);
     addAutoReleasedObject(new DesktopQtVersionFactory);
-    addAutoReleasedObject(new SimulatorQtVersionFactory);
     addAutoReleasedObject(new WinCeQtVersionFactory);
     addAutoReleasedObject(new UiCodeModelManager);
 
@@ -102,7 +100,7 @@ bool QtSupportPlugin::initialize(const QStringList &arguments, QString *errorMes
 
 static QString qmakeProperty(const char *propertyName)
 {
-    ProjectExplorer::Project *project = ProjectExplorer::ProjectExplorerPlugin::currentProject();
+    ProjectExplorer::Project *project = ProjectExplorer::ProjectTree::currentProject();
     if (!project || !project->activeTarget())
         return QString();
 
@@ -121,8 +119,8 @@ void QtSupportPlugin::extensionsInitialized()
         []() { return qmakeProperty("QT_HOST_BINS"); });
 
     expander->registerVariable(kInstallBins,
-        tr("Full path to the target bin directory of the current project's Qt version."
-           " You probably want %1 instead.").arg(QString::fromLatin1(kHostBins)),
+        tr("Full path to the target bin directory of the current project's Qt version.<br>"
+           "You probably want %1 instead.").arg(QString::fromLatin1(kHostBins)),
         []() { return qmakeProperty("QT_INSTALL_BINS"); });
 }
 
