@@ -48,9 +48,7 @@ class SubversionClient : public VcsBase::VcsBaseClient
     Q_OBJECT
 
 public:
-    SubversionClient(SubversionSettings *settings);
-
-    SubversionSettings *settings() const;
+    SubversionClient();
 
     VcsBase::VcsCommand *createCommitCmd(const QString &repositoryRoot,
                                          const QStringList &files,
@@ -59,28 +57,28 @@ public:
     void commit(const QString &repositoryRoot,
                 const QStringList &files,
                 const QString &commitMessageFile,
-                const QStringList &extraOptions = QStringList());
+                const QStringList &extraOptions = QStringList()) override;
 
     void diff(const QString &workingDirectory, const QStringList &files,
-              const QStringList &extraOptions);
+              const QStringList &extraOptions) override;
 
     void log(const QString &workingDir,
              const QStringList &files = QStringList(),
              const QStringList &extraOptions = QStringList(),
-             bool enableAnnotationContextMenu = false) Q_DECL_OVERRIDE;
+             bool enableAnnotationContextMenu = false) override;
 
     void describe(const QString &workingDirectory, int changeNumber, const QString &title);
-    QString findTopLevelForFile(const QFileInfo &file) const;
-    QStringList revisionSpec(const QString &revision) const;
-    StatusItem parseStatusLine(const QString &line) const;
+    QString findTopLevelForFile(const QFileInfo &file) const override;
+    QStringList revisionSpec(const QString &revision) const override;
+    StatusItem parseStatusLine(const QString &line) const override;
 
     // Add authorization options to the command line arguments.
-    static QStringList addAuthenticationOptions(const SubversionSettings &settings);
+    static QStringList addAuthenticationOptions(const VcsBase::VcsBaseClientSettings &settings);
 
     QString synchronousTopic(const QString &repository);
 
 protected:
-    Core::Id vcsEditorKind(VcsCommandTag cmd) const;
+    Core::Id vcsEditorKind(VcsCommandTag cmd) const override;
 
 private:
     DiffController *findOrCreateDiffEditor(const QString &documentId, const QString &source,
